@@ -3,10 +3,10 @@
  *  Module    : proto.h
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   :
- *  Updated   : 2009-12-09
+ *  Updated   : 2016-07-29
  *  Notes     :
  *
- * Copyright (c) 1997-2010 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2016 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,7 +56,7 @@ extern char group_flag(char ch);
 extern int find_newnews_index(const char *cur_newnews_host);
 extern int read_news_active_file(void);
 extern t_bool match_group_list(const char *group, const char *group_list);
-extern t_bool parse_active_line(char *line, long *max, long *min, char *moderated);
+extern t_bool parse_active_line(char *line, t_artnum *max, t_artnum *min, char *moderated);
 extern t_bool process_bogus(char *name);
 extern t_bool need_reread_active_file(void);
 extern t_bool resync_active_file(void);
@@ -64,6 +64,7 @@ extern void create_save_active_file(void);
 extern void load_newnews_info(char *info);
 
 /* art.c */
+extern int find_artnum(t_artnum art);
 extern int global_get_multipart_info(int aindex, MultiPartInfo *setme);
 extern t_bool index_group(struct t_group *group);
 extern void do_update(t_bool catchup);
@@ -86,12 +87,12 @@ extern void write_attributes_file(const char *file);
 #endif /* NNTP_ABLE */
 
 /* charset.c */
-extern char *convert_to_printable(char *buf);
+extern char *convert_to_printable(char *buf, t_bool keep_tab);
 extern t_bool is_art_tex_encoded(FILE *fp);
 extern void convert_iso2asc(char *iso, char **asc_buffer, size_t *max_line_len, int t);
 extern void convert_tex2iso(char *from, char *to);
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	extern wchar_t *wconvert_to_printable(wchar_t *wbuf);
+	extern wchar_t *wconvert_to_printable(wchar_t *wbuf, t_bool keep_tab);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 /* color.c */
@@ -110,7 +111,7 @@ extern char *quote_space_to_dash(char *str);
 extern const char *print_boolean(t_bool value);
 extern t_bool match_boolean(char *line, const char *pat, t_bool *dst);
 extern t_bool match_integer(char *line, const char *pat, int *dst, int maxval);
-extern t_bool match_list(char *line, constext *pat, constext *const *table, size_t tablelen, int *dst);
+extern t_bool match_list(char *line, constext *pat, constext *const *table, int *dst);
 extern t_bool match_long(char *line, const char *pat, long *dst);
 extern t_bool match_string(char *line, const char *pat, char *dst, size_t dstlen);
 extern t_bool read_config_file(char *file, t_bool global_file);
@@ -120,8 +121,8 @@ extern void write_config_file(char *file);
 
 /* cook.c */
 extern const char *get_filename(t_param *ptr);
-extern t_bool cook_article(t_bool wrap_lines, t_openartinfo *artinfo, int hide_uue);
-extern t_bool expand_ctrl_chars(char **line, size_t *length, size_t cook_width);
+extern t_bool cook_article(t_bool wrap_lines, t_openartinfo *artinfo, int hide_uue, t_bool show_all_headers);
+extern t_bool expand_ctrl_chars(char **line, size_t *length, size_t lcook_width);
 
 /* curses.c */
 extern OUTC_RETTYPE outchar(OUTC_ARGS);
@@ -129,7 +130,9 @@ extern int InitScreen(void);
 extern int RawState(void);
 extern int ReadCh(void);
 extern int get_arrow_key(int prech);
-extern int get_termcaps(void);
+#if defined(M_UNIX) && !defined(USE_CURSES)
+	extern int get_termcaps(void);
+#endif /* M_UNIX && !USE_CURSES */
 extern void EndInverse(void);
 extern void EndWin(void);
 extern void InitWin(void);
@@ -154,6 +157,9 @@ extern void word_highlight_string(int row, int col, int size, int color);
 	extern void MoveCursor(int row, int col);
 	extern void ScrollScreen(int lines_to_scroll);
 	extern void SetScrollRegion(int topline, int bottomline);
+#	ifdef HAVE_COLOR
+		extern void reset_screen_attr(void);
+#	endif /* HAVE_COLOR */
 #endif /* USE_CURSES */
 #if 0
 	extern void ToggleInverse(void);
@@ -170,6 +176,9 @@ extern void word_highlight_string(int row, int col, int size, int color);
 	extern void debug_print_filters(void);
 	extern void debug_print_header(struct t_article *s);
 	extern void debug_print_malloc(int is_malloc, const char *xfile, int line, size_t size);
+#	ifdef DEBUG
+	extern const char *logtime(void);
+#	endif /* DEBUG */
 #endif /* DEBUG */
 
 /* envarg.c */
@@ -185,8 +194,10 @@ extern t_bool quick_filter(t_function type, struct t_group *group, struct t_arti
 extern t_bool quick_filter_select_posted_art(struct t_group *group, const char *subj, const char *a_message_id);
 extern t_bool read_filter_file(const char *file);
 extern void free_filter_array(struct t_filters *ptr);
-extern void refresh_filter_menu(void);
-extern void unfilter_articles(void);
+#if defined(SIGWINCH) || defined(SIGTSTP)
+	extern void refresh_filter_menu(void);
+#endif /* SIGWINCH || SIGTSTP */
+extern void unfilter_articles(struct t_group *group);
 extern void write_filter_file(const char *filename);
 
 /* getline.c */
@@ -208,7 +219,6 @@ extern void scroll_up(void);
 extern void set_first_screen_item(void);
 
 /* group.c */
-extern int find_new_pos(int old_top, long old_artnum, int cur_pos);
 extern int group_page(struct t_group *group);
 extern t_bool group_mark_postprocess(int function, t_function feed_type, int respnum);
 extern void clear_note_area(void);
@@ -234,6 +244,11 @@ extern const char *get_host_name(void);
 	extern char *build_sender(void);
 #endif /* !FORGERY */
 
+/* heapsort.c */
+#ifndef HAVE_HEAPSORT
+	extern int heapsort(void *, size_t, size_t, t_compfunc);
+#endif /* !HAVE_HEAPSORT */
+
 /* inews.c */
 extern t_bool submit_news_file(char *name, struct t_group *group, char *a_message_id);
 extern void get_from_name(char *from_name, struct t_group *thisgrp);
@@ -243,7 +258,7 @@ extern void get_user_info(char *user_name, char *full_name);
 extern void init_selfinfo(void);
 extern void postinit_regexp(void);
 #ifdef HAVE_COLOR
-	extern void postinit_colors(void);
+	extern void postinit_colors(int last_color);
 #endif /* HAVE_COLOR */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	extern t_bool utf8_pcre(void);
@@ -282,9 +297,9 @@ extern t_bool dot_unlock(const char *filename);
 
 /* mail.c */
 extern t_bool art_edit(struct t_group *group, struct t_article *article);
-extern void find_art_max_min(const char *group_path, long *art_max, long *art_min);
+extern void find_art_max_min(const char *group_path, t_artnum *art_max, t_artnum *art_min);
 extern void print_active_head(const char *active_file);
-extern void print_group_line(FILE *fp, const char *group_name, long art_max, long art_min, const char *base_dir);
+extern void print_group_line(FILE *fp, const char *group_name, t_artnum art_max, t_artnum art_min, const char *base_dir);
 extern void read_descriptions(t_bool verb);
 extern void grp_del_mail_arts(struct t_group *group);
 extern void grp_del_mail_art(struct t_article *article);
@@ -301,6 +316,7 @@ extern void giveup(void);
 /* memory.c */
 extern void expand_active(void);
 extern void expand_art(void);
+extern void expand_base(void);
 extern void expand_newnews(void);
 extern void expand_save(void);
 extern void expand_scope(void);
@@ -333,15 +349,15 @@ extern const char *get_val(const char *env, const char *def);
 extern const char *gnksa_strerror(int errcode);
 extern int gnksa_check_from(char *from);
 extern int gnksa_split_from(const char *from, char *address, char *realname, int *addrtype);
-extern int get_initials(int respnum, char *s, int maxsize);
+extern int get_initials(struct t_article *art, char *s, int maxsize);
 extern int gnksa_do_check_from(const char *from, char *address, char *realname);
-extern int my_chdir(char *path);
 extern int my_mkdir(char *path, mode_t mode);
 extern int parse_from(const char *from, char *address, char *realname);
 extern int strfmailer(const char *mail_prog, char *subject, char *to, const char *filename, char *dest, size_t maxsize, const char *format);
 extern int strfpath(const char *format, char *str, size_t maxsize, struct t_group *group, t_bool expand_all);
 extern int strfquote(const char *group, int respnum, char *s, size_t maxsize, char *format);
 extern int tin_version_info(FILE *fp);
+extern int tin_gettime(struct t_tintime *tt);
 extern long file_mtime(const char *file);
 extern long file_size(const char *file);
 extern t_bool backup_file(const char *filename, const char *backupname);
@@ -351,7 +367,7 @@ extern t_bool invoke_editor(const char *filename, int lineno, struct t_group *gr
 extern t_bool mail_check(void);
 extern void append_file(char *old_filename, char *new_filename);
 extern void asfail(const char *file, int line, const char *cond);
-extern void base_name(const char *fullpath, char *program);
+extern void base_name(const char *fullpath, char *file);
 extern void cleanup_tmp_files(void);
 extern void copy_body(FILE *fp_ip, FILE *fp_op, char *prefix, char *initl, t_bool raw_data);
 extern void create_index_lock_file(char *the_lock_file);
@@ -367,7 +383,7 @@ extern void read_input_history_file(void);
 extern void rename_file(const char *old_filename, const char *new_filename);
 extern void show_inverse_video_status(void);
 extern void strip_name(const char *from, char *address);
-extern void tin_done(int ret);
+extern void tin_done(int ret, const char *fmt, ...);
 extern void toggle_inverse_video(void);
 #if defined(CHARSET_CONVERSION) || (defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE))
 	extern char *utf8_valid(char *line);
@@ -394,7 +410,7 @@ extern void toggle_inverse_video(void);
 #endif /* !NO_SHELL_ESCAPE */
 
 /* newsrc.c */
-extern int group_get_art_info(char *tin_spooldir, char *groupname, int grouptype, long *art_count, long *art_max, long *art_min);
+extern int group_get_art_info(char *tin_spooldir, char *groupname, int grouptype, t_artnum *art_count, t_artnum *art_max, t_artnum *art_min);
 extern signed long int read_newsrc(char *newsrc_file, t_bool allgroups);
 extern signed long int write_newsrc(void);
 extern t_bool pos_group_in_newsrc(struct t_group *group, int pos);
@@ -402,10 +418,10 @@ extern void art_mark(struct t_group *group, struct t_article *art, int flag);
 extern void backup_newsrc(void);
 extern void catchup_newsrc_file(void);
 extern void delete_group(char *group);
-extern void expand_bitmap(struct t_group *group, long min);
+extern void expand_bitmap(struct t_group *group, t_artnum min);
 extern void grp_mark_read(struct t_group *group, struct t_article *art);
 extern void grp_mark_unread(struct t_group *group);
-extern void parse_unread_arts(struct t_group *group, long min);
+extern void parse_unread_arts(struct t_group *group, t_artnum min);
 extern void reset_newsrc(void);
 extern void subscribe(struct t_group *group, int sub_state, t_bool get_info);
 extern void thd_mark_read(struct t_group *group, long thread);
@@ -436,7 +452,7 @@ extern t_bool get_newsrcname(char *newsrc_name, size_t newsrc_name_len, const ch
 
 /* options_menu.c */
 extern char *fmt_option_prompt(char *dst, size_t len, t_bool editing, enum option_enum option);
-extern void config_page(const char *grpname);
+extern void config_page(const char *grpname, enum context level);
 extern int option_row(enum option_enum option);
 extern t_bool option_is_visible(enum option_enum option);
 extern void check_score_defaults(void);
@@ -444,7 +460,7 @@ extern void refresh_config_page(enum option_enum act_option);
 extern void show_menu_help(const char *help_message);
 
 /* page.c */
-extern int show_page(struct t_group *group, int respnum, int *threadnum);
+extern int show_page(struct t_group *group, int start_respnum, int *threadnum);
 extern void display_info_page(int part);
 extern void draw_page(const char *group, int part);
 extern void info_pager(FILE *info_fh, const char *title, t_bool wrap_at_ends);
@@ -459,7 +475,7 @@ extern time_t parsedate(char *p, TIMEINFO *now);
 	extern t_bool pgp_check_article(t_openartinfo *artinfo);
 	extern void init_pgp(void);
 	extern void invoke_pgp_mail(const char *nam, char *mail_to);
-	extern void invoke_pgp_news(char *the_article);
+	extern void invoke_pgp_news(char *artfile);
 #endif /* HAVE_PGP_GPG */
 
 /* plp_snprintf.c */
@@ -471,6 +487,7 @@ extern time_t parsedate(char *p, TIMEINFO *now);
 #endif /* !HAVE_VSNPRINTF */
 
 /* post.c */
+extern char *backup_article_name(const char *the_article);
 extern char *checknadd_headers(const char *infile, struct t_group *group);
 extern int count_postponed_articles(void);
 extern int mail_to_author(const char *group, int respnum, t_bool copy_text, t_bool with_headers, t_bool raw_data);
@@ -485,6 +502,9 @@ extern t_bool reread_active_after_posting(void);
 extern t_bool user_posted_messages(void);
 extern void init_postinfo(void);
 extern void quick_post_article(t_bool postponed_only);
+#if defined(SIGWINCH) || defined(SIGTSTP)
+	extern void refresh_post_screen(int context);
+#endif /* SIGWINCH || SIGTSTP */
 #ifdef USE_CANLOCK
 	extern char *build_canlock(const char *messageid, const char *secret);
 	extern char *get_secret(void);
@@ -538,12 +558,13 @@ extern int read_decoded_qp_line(FILE *file, char **line, size_t *max_line_len, c
 extern void rfc1521_encode(char *line, FILE *f, int e);
 
 /* rfc2046.c */
-extern FILE *open_art_fp(struct t_group *group, long art);
+extern FILE *open_art_fp(struct t_group *group, t_artnum art);
 extern const char *get_param(t_param *list, const char *name);
-extern char *parse_header(char *buf, const char *pat, t_bool decode, t_bool structured);
-extern int art_open(t_bool wrap_lines, struct t_article *art, struct t_group *group, t_openartinfo *artinfo, t_bool show_progress_meter, char *pmesg);
+extern char *parse_header(char *buf, const char *pat, t_bool decode, t_bool structured, t_bool keep_tab);
+extern int art_open(t_bool wrap_lines, struct t_article *art, struct t_group *group, t_openartinfo *artinfo, t_bool show_progress_meter, const char *pmesg);
 extern int content_type(char *type);
 extern int parse_rfc822_headers(struct t_header *hdr, FILE *from, FILE *to);
+extern t_param *new_params(void);
 extern t_part *new_part(t_part *part);
 extern void art_close(t_openartinfo *artinfo);
 extern void free_and_init_header(struct t_header *hdr);
@@ -562,10 +583,13 @@ extern void compose_mail_text_plain(const char *filename, struct t_group *group)
 /* save.c */
 extern int check_start_save_any_news(int function, t_bool catchup);
 extern t_bool create_path(const char *path);
-extern t_bool post_process_files(t_function proc_type_type, t_bool auto_delete);
+extern t_bool post_process_files(t_function proc_type_func, t_bool auto_delete);
 extern t_bool save_and_process_art(t_openartinfo *artinfo, struct t_article *artptr, t_bool is_mailbox, const char *inpath, int max, t_bool post_process);
+extern t_part *get_part(int n);
+extern t_url *find_url(int n);
+extern void attachment_page(t_openartinfo *art);
 extern void decode_save_mime(t_openartinfo *art, t_bool postproc);
-extern void print_art_seperator_line(FILE *fp, t_bool is_mailbox);
+extern void print_art_separator_line(FILE *fp, t_bool is_mailbox);
 
 /* screen.c */
 extern char *fmt_message(const char *fmt, va_list ap);
@@ -577,7 +601,7 @@ extern void error_message(unsigned int sdelay, const char *fmt, ...);
 extern void info_message(const char *fmt, ...);
 extern void perror_message(const char *fmt, ...);
 extern void ring_bell(void);
-extern void show_progress(const char *txt, long count, long total);
+extern void show_progress(const char *txt, t_artnum count, t_artnum total);
 extern void show_title(const char *title);
 extern void spin_cursor(void);
 extern void stow_cursor(void);
@@ -590,6 +614,7 @@ extern int search(t_function func, int current_art, t_bool repeat);
 extern int search_active(t_bool forward, t_bool repeat);
 extern int search_article(t_bool forward, t_bool repeat, int start_line, int lines, t_lineinfo *line, int reveal_ctrl_l_lines, FILE *fp);
 extern int search_body(struct t_group *group, int current_art, t_bool repeat);
+extern int generic_search(t_bool forward, t_bool repeat, int current, int last, int level);
 extern void reset_srch_offsets(void);
 
 /* select.c */
@@ -601,7 +626,7 @@ extern void show_selection_page(void);
 extern void toggle_my_groups(const char *group);
 
 /* sigfile.c */
-extern void msg_write_signature(FILE *fp, t_bool flag, struct t_group *thisgroup);
+extern void msg_write_signature(FILE *fp, t_bool include_dot_signature, struct t_group *thisgroup);
 
 /* signal.c */
 extern RETSIGTYPE(*sigdisp (int signum, RETSIGTYPE (*func)(SIG_ARGS))) (SIG_ARGS);
@@ -621,23 +646,25 @@ extern char *fmt_string(const char *fmt, ...);
 extern char *my_strdup(const char *str);
 extern char *str_trim(char *string);
 extern char *strunc(const char *message, int len);
-extern char *tin_ltoa(long value, int digits);
+extern char *tin_ltoa(t_artnum value, int digits);
 extern char *tin_strtok(char *str, const char *delim);
 extern int sh_format(char *dst, size_t len, const char *fmt, ...);
+extern int strwidth(const char *str);
 extern size_t mystrcat(char **t, const char *s);
 extern void my_strncpy(char *p, const char *q, size_t n);
+extern void parse_format_string(const char *fmtstr, struct t_fmt *fmt);
 extern void str_lwr(char *str);
 #if !defined(HAVE_STRCASESTR) || defined(DECL_STRCASESTR)
-	extern const char *strcasestr(const char *haystack, const char *needle);
+	extern char *strcasestr(const char *haystack, const char *needle);
 #endif /* !HAVE_STRCASESTR || DECL_STRCASESTR */
 #if !defined(HAVE_STRSEP) || defined(DECL_SEP)
 	extern char *strsep(char **stringp, const char *delim);
 #endif /* !HAVE_STRSEP || DECL_STRSEP */
 #ifndef HAVE_STRPBRK
-	extern char *strpbrk(char *str1, char *str2);
+	extern char *strpbrk(const char *str1, const char *str2);
 #endif /* !HAVE_STRPBRK */
 #ifndef HAVE_STRSTR
-	extern char *strstr(char *text, char *pattern);
+	extern char *strstr(const char *text, const char *pattern);
 #endif /* !HAVE_STRSTR */
 #ifndef HAVE_STRCASECMP
 	extern int strcasecmp(const char *p, const char *q);
@@ -663,10 +690,16 @@ extern void str_lwr(char *str);
 #	define strrstr(s,p)	my_strrstr(s,p)
 #endif /* !HAVE_STRRSTR */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	extern char *spart(const char *str, int columns, t_bool pad);
 	extern char *wchar_t2char(const wchar_t *wstr);
+	extern wchar_t *abbr_wcsgroupname(const wchar_t *grpname, int len);
 	extern wchar_t *char2wchar_t(const char *str);
 	extern wchar_t *wcspart(const wchar_t *wstr, int columns, t_bool pad);
+	extern wchar_t *wexpand_tab(wchar_t *wstr, size_t tab_width);
 	extern wchar_t *wstrunc(const wchar_t *wmessage, int len);
+#else
+	extern char *abbr_groupname(const char *grpname, size_t len);
+	extern char *expand_tab(char *str, size_t tab_width);
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && defined(HAVE_UNICODE_UBIDI_H) && !defined(NO_LOCALE)
 	extern char *render_bidi(const char *str, t_bool *is_rtl);
@@ -674,6 +707,10 @@ extern void str_lwr(char *str);
 #ifdef HAVE_UNICODE_NORMALIZATION
 	extern char *normalize(const char *str);
 #endif /* HAVE_UNICODE_NORMALIZATION */
+#if defined(HAVE_LIBICUUC) && defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	extern UChar *char2UChar(const char *str);
+	extern char *UChar2char(const UChar *ustr);
+#endif /* HAVE_LIBICUUC && MULTIBYTE_ABLE && !NO_LOCALE */
 
 /* tags.c */
 extern int line_is_tagged(int n);
@@ -693,7 +730,7 @@ extern void untag_article(long art);
 #endif /* !HAVE_TMPFILE */
 
 /* my_tmpfile.c */
-extern int my_tmpfile(char *filename, size_t name_size, t_bool need_name, const char *base_dir);
+extern int my_tmpfile(char *filename, size_t name_size, const char *base_dir);
 
 /* thread.c */
 extern int find_response(int i, int n);
@@ -732,8 +769,8 @@ extern t_bool wildmatpos(const char *text, char *p, t_bool icase, int *srch_offs
 
 /* xref.c */
 extern t_bool overview_xref_support(void);
-extern void NSETRNG0(t_bitmap *bitmap, long low, long high);
-extern void NSETRNG1(t_bitmap *bitmap, long low, long high);
+extern void NSETRNG0(t_bitmap *bitmap, t_artnum low, t_artnum high);
+extern void NSETRNG1(t_bitmap *bitmap, t_artnum low, t_artnum high);
 extern void art_mark_xref_read(struct t_article *art);
 
 #endif /* !PROTO_H */
