@@ -40,10 +40,10 @@
 
 #	define PRODUCT		"tin"
 #	ifndef TIN_AUTOCONF_H
-#		define VERSION		"2.3.4"
+#		define VERSION		"2.4.0"
 #	endif /* !TIN_AUTOCONF_H */
-#	define RELEASEDATE	"20160628"
-#	define RELEASENAME	"Newton"
+#	define RELEASEDATE	"20160823"
+#	define RELEASENAME	"Octomore"
 /* config-file versions - must by dotted triples */
 #	define TINRC_VERSION	"1.3.13"
 #	define ATTRIBUTES_VERSION	"1.0.8"
