@@ -3,7 +3,7 @@
  *  Module    : active.c
  *  Author    : I. Lea
  *  Created   : 1992-02-16
- *  Updated   : 2016-06-02
+ *  Updated   : 2016-07-29
  *  Notes     :
  *
  * Copyright (c) 1992-2016 Iain Lea <iain@bricbrac.de>
@@ -440,8 +440,7 @@ do_read_newsrc_active_file(
 						continue;
 
 					case ERR_ACCESS:
-						error_message(2, "%s%s", cCRLF, line);
-						tin_done(NNTP_ERROR_EXIT);
+						tin_done(NNTP_ERROR_EXIT, "%s", line);
 						/* keep lint quiet: */
 						/* FALLTHROUGH */
 
@@ -538,8 +537,7 @@ read_newsrc_active_file(
 #ifdef NNTP_ABLE
 	if (need_auth) { /* delayed auth */
 		if (!authenticate(nntp_server, userid, FALSE) || do_read_newsrc_active_file(fp)) {
-			error_message(2, _(txt_auth_failed), ERR_ACCESS);
-			tin_done(EXIT_FAILURE);
+			tin_done(EXIT_FAILURE, _(txt_auth_failed), ERR_ACCESS);
 		}
 	}
 #endif /* NNTP_ABLE */
@@ -551,10 +549,9 @@ read_newsrc_active_file(
 	 */
 	if (tin_errno || !num_active) {
 		if (newsrc_active && !num_active)
-			error_message(2, _(txt_error_server_has_no_listed_groups), newsrc);
+			tin_done(EXIT_FAILURE, _(txt_error_server_has_no_listed_groups), newsrc);
 		else
-			error_message(2, _(txt_active_file_is_empty), (read_news_via_nntp ? (read_saved_news ? news_active_file : _(txt_servers_active)) : news_active_file));
-		tin_done(EXIT_FAILURE);
+			tin_done(EXIT_FAILURE, _(txt_active_file_is_empty), (read_news_via_nntp ? (read_saved_news ? news_active_file : _(txt_servers_active)) : news_active_file));
 	}
 
 	if (!batch_mode || verbose)
@@ -600,16 +597,14 @@ read_active_file(
 
 #ifdef NNTP_ABLE
 		if (read_news_via_nntp)
-			error_message(2, _(txt_cannot_retrieve), ACTIVE_FILE);
+			tin_done(EXIT_FAILURE, _(txt_cannot_retrieve), ACTIVE_FILE);
 #	ifndef NNTP_ONLY
 		else
-			error_message(2, _(txt_cannot_open_active_file), news_active_file, tin_progname);
+			tin_done(EXIT_FAILURE, _(txt_cannot_open_active_file), news_active_file, tin_progname);
 #	endif /* !NNTP_ONLY */
 #else
-		error_message(2, _(txt_cannot_open), news_active_file);
+		tin_done(EXIT_FAILURE, _(txt_cannot_open), news_active_file);
 #endif /* NNTP_ABLE */
-
-		tin_done(EXIT_FAILURE);
 	}
 
 	while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
@@ -657,10 +652,8 @@ read_active_file(
 	/*
 	 * Exit if active file wasn't read correctly or is empty
 	 */
-	if (tin_errno || !num_active) {
-		error_message(2, _(txt_active_file_is_empty), (read_news_via_nntp ? (read_saved_news ? news_active_file : _(txt_servers_active)) : news_active_file));
-		tin_done(EXIT_FAILURE);
-	}
+	if (tin_errno || !num_active)
+		tin_done(EXIT_FAILURE, _(txt_active_file_is_empty), (read_news_via_nntp ? (read_saved_news ? news_active_file : _(txt_servers_active)) : news_active_file));
 
 	if (!batch_mode || verbose)
 		my_fputc('\n', stdout);
@@ -689,8 +682,7 @@ read_active_counts(
 		if (cmd_line && !batch_mode)
 			my_fputc('\n', stderr);
 
-		error_message(2, _(txt_cannot_retrieve), ACTIVE_FILE);
-		tin_done(EXIT_FAILURE);
+		tin_done(EXIT_FAILURE,_(txt_cannot_retrieve), ACTIVE_FILE);
 	}
 
 	while ((ptr = tin_fgets(fp, FALSE)) != NULL) {
@@ -736,10 +728,8 @@ read_active_counts(
 	/*
 	 * Exit if active file wasn't read correctly or is empty
 	 */
-	if (tin_errno || !num_active) {
-		error_message(2, _(txt_active_file_is_empty), _(txt_servers_active));
-		tin_done(EXIT_FAILURE);
-	}
+	if (tin_errno || !num_active)
+		tin_done(EXIT_FAILURE, _(txt_active_file_is_empty), _(txt_servers_active));
 
 	if (!batch_mode || verbose)
 		my_fputc('\n', stdout);
@@ -890,10 +880,8 @@ read_news_active_file(
 						}
 					}
 					if (need_auth) { /* retry after auth is overkill here, so just auth */
-						if (!authenticate(nntp_server, userid, FALSE)) {
-							error_message(2, _(txt_auth_failed), nntp_caps.type == CAPABILITIES ? ERR_AUTHFAIL : ERR_ACCESS);
-							tin_done(EXIT_FAILURE);
-						}
+						if (!authenticate(nntp_server, userid, FALSE))
+							tin_done(EXIT_FAILURE, _(txt_auth_failed), nntp_caps.type == CAPABILITIES ? ERR_AUTHFAIL : ERR_ACCESS);
 					}
 				}
 				did_reconnect = FALSE;
