@@ -3,10 +3,10 @@
  *  Module    : help.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2009-09-27
+ *  Updated   : 2016-06-06
  *  Notes     :
  *
- * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2016 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -56,6 +56,49 @@ static void make_help_page(FILE *fp, const t_help_page *helppage, const struct k
 
 static constext txt_help_empty_line[] = "";
 
+static t_help_page attachment_help_page[] = {
+	{ txt_help_title_navi, NOT_ASSIGNED },
+	{ txt_help_global_page_down, GLOBAL_PAGE_DOWN },
+	{ txt_help_global_page_up, GLOBAL_PAGE_UP },
+	{ txt_help_global_line_down, GLOBAL_LINE_DOWN },
+	{ txt_help_global_line_up, GLOBAL_LINE_UP },
+	{ txt_help_global_scroll_down, GLOBAL_SCROLL_DOWN },
+	{ txt_help_global_scroll_up, GLOBAL_SCROLL_UP },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_attachment_first, GLOBAL_FIRST_PAGE },
+	{ txt_help_attachment_last, GLOBAL_LAST_PAGE },
+	{ txt_help_attachment_goto, NOT_ASSIGNED },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_disp, NOT_ASSIGNED },
+	{ txt_help_attachment_toggle_info_line, GLOBAL_TOGGLE_INFO_LAST_LINE },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_attachment_ops, NOT_ASSIGNED },
+	{ txt_help_attachment_select, ATTACHMENT_SELECT },
+#ifndef DONT_HAVE_PIPING
+	{ txt_help_attachment_pipe, ATTACHMENT_PIPE },
+	{ txt_help_attachment_pipe_raw, GLOBAL_PIPE },
+#endif /* !DONT_HAVE_PIPING */
+	{ txt_help_attachment_save, ATTACHMENT_SAVE },
+	{ txt_help_attachment_tag, ATTACHMENT_TAG },
+	{ txt_help_attachment_tag_pattern, ATTACHMENT_TAG_PATTERN },
+	{ txt_help_attachment_toggle_tagged, ATTACHMENT_TOGGLE_TAGGED },
+	{ txt_help_attachment_untag, ATTACHMENT_UNTAG },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_attachment_search_forwards, GLOBAL_SEARCH_SUBJECT_FORWARD },
+	{ txt_help_attachment_search_backwards, GLOBAL_SEARCH_SUBJECT_BACKWARD },
+	{ txt_help_global_search_repeat, GLOBAL_SEARCH_REPEAT },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_misc, NOT_ASSIGNED },
+	{ txt_help_select_quit, GLOBAL_QUIT },
+	{ txt_help_global_help, GLOBAL_HELP },
+	{ txt_help_global_toggle_mini_help, GLOBAL_TOGGLE_HELP_DISPLAY },
+	{ txt_help_global_redraw_screen, GLOBAL_REDRAW_SCREEN },
+#ifndef NO_SHELL_ESCAPE
+	{ txt_help_global_shell_escape, GLOBAL_SHELL_ESCAPE },
+#endif /* !NO_SHELL_ESCAPE */
+	{ NULL, NOT_ASSIGNED }
+};
+
 static t_help_page attrib_help_page[] = {
 	{ txt_help_title_navi, NOT_ASSIGNED },
 	{ txt_help_global_page_down, GLOBAL_PAGE_DOWN },
@@ -75,15 +118,18 @@ static t_help_page attrib_help_page[] = {
 	{ txt_help_global_search_repeat, GLOBAL_SEARCH_REPEAT },
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_title_attrib_ops, NOT_ASSIGNED },
-	{ txt_help_attrib_reset_attrib , CONFIG_RESET_ATTRIB },
-	{ txt_help_attrib_select , CONFIG_SELECT },
-	{ txt_help_attrib_toggle_attrib , CONFIG_TOGGLE_ATTRIB },
+	{ txt_help_attrib_reset_attrib, CONFIG_RESET_ATTRIB },
+	{ txt_help_attrib_select, CONFIG_SELECT },
+	{ txt_help_attrib_toggle_attrib, CONFIG_TOGGLE_ATTRIB },
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_title_misc, NOT_ASSIGNED },
 	{ txt_help_select_quit, GLOBAL_QUIT },
 	{ txt_help_select_quit_no_write, CONFIG_NO_SAVE },
 	{ txt_help_global_help, GLOBAL_HELP },
 	{ txt_help_global_redraw_screen, GLOBAL_REDRAW_SCREEN },
+#ifndef NO_SHELL_ESCAPE
+	{ txt_help_global_shell_escape, GLOBAL_SHELL_ESCAPE },
+#endif /* !NO_SHELL_ESCAPE */
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_global_version, GLOBAL_VERSION },
 	{ NULL, NOT_ASSIGNED }
@@ -108,15 +154,18 @@ static t_help_page config_help_page[] = {
 	{ txt_help_global_search_repeat, GLOBAL_SEARCH_REPEAT },
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_title_config_ops, NOT_ASSIGNED },
-	{ txt_help_config_select , CONFIG_SELECT },
-	{ txt_help_config_toggle_attrib , CONFIG_TOGGLE_ATTRIB },
-	{ txt_help_config_scope_menu , CONFIG_SCOPE_MENU },
+	{ txt_help_config_select, CONFIG_SELECT },
+	{ txt_help_config_toggle_attrib, CONFIG_TOGGLE_ATTRIB },
+	{ txt_help_config_scope_menu, CONFIG_SCOPE_MENU },
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_title_misc, NOT_ASSIGNED },
 	{ txt_help_select_quit, GLOBAL_QUIT },
 	{ txt_help_select_quit_no_write, CONFIG_NO_SAVE },
 	{ txt_help_global_help, GLOBAL_HELP },
 	{ txt_help_global_redraw_screen, GLOBAL_REDRAW_SCREEN },
+#ifndef NO_SHELL_ESCAPE
+	{ txt_help_global_shell_escape, GLOBAL_SHELL_ESCAPE },
+#endif /* !NO_SHELL_ESCAPE */
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_global_version, GLOBAL_VERSION },
 	{ NULL, NOT_ASSIGNED }
@@ -149,6 +198,9 @@ static t_help_page scope_help_page[] = {
 	{ txt_help_global_help, GLOBAL_HELP },
 	{ txt_help_global_toggle_mini_help, GLOBAL_TOGGLE_HELP_DISPLAY },
 	{ txt_help_global_redraw_screen, GLOBAL_REDRAW_SCREEN },
+#ifndef NO_SHELL_ESCAPE
+	{ txt_help_global_shell_escape, GLOBAL_SHELL_ESCAPE },
+#endif /* !NO_SHELL_ESCAPE */
 	{ NULL, NOT_ASSIGNED }
 };
 
@@ -166,6 +218,10 @@ static t_help_page select_help_page[] = {
 	{ txt_help_select_group_by_num, NOT_ASSIGNED },
 	{ txt_help_select_goto_group, SELECT_GOTO },
 	{ txt_help_select_next_unread_group, SELECT_NEXT_UNREAD_GROUP },
+#ifdef NNTP_ABLE
+	{ txt_help_select_lookup_group, GLOBAL_LOOKUP_MESSAGEID },
+	{ txt_help_select_lookup_group_comment, NOT_ASSIGNED },
+#endif /* NNTP_ABLE */
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_select_search_group_forwards, GLOBAL_SEARCH_SUBJECT_FORWARD },
 	{ txt_help_select_search_group_backwards, GLOBAL_SEARCH_SUBJECT_BACKWARD },
@@ -276,6 +332,7 @@ static t_help_page group_help_page[] = {
 	{ txt_help_global_post, GLOBAL_POST },
 	{ txt_help_global_post_postponed, GLOBAL_POSTPONED },
 	{ txt_help_article_repost, GROUP_REPOST },
+	{ txt_help_article_cancel, GROUP_CANCEL },
 #endif /* NO_POSTING */
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_global_article_range, GLOBAL_SET_RANGE },
@@ -371,6 +428,7 @@ static t_help_page thread_help_page[] = {
 #ifndef NO_POSTING
 	{ txt_help_global_post, GLOBAL_POST },
 	{ txt_help_global_post_postponed, GLOBAL_POSTPONED },
+	{ txt_help_article_cancel, THREAD_CANCEL },
 #endif /* NO_POSTING */
 	{ txt_help_empty_line, NOT_ASSIGNED },
 	{ txt_help_global_article_range, GLOBAL_SET_RANGE },
@@ -460,7 +518,8 @@ static t_help_page page_help_page[] = {
 	{ txt_help_global_toggle_info_line, GLOBAL_TOGGLE_INFO_LAST_LINE },
 	{ txt_help_article_toggle_rot13, PAGE_TOGGLE_ROT13 },
 	{ txt_help_global_toggle_inverse_video, GLOBAL_TOGGLE_INVERSE_VIDEO },
-	{ txt_help_article_show_raw, PAGE_TOGGLE_HEADERS },
+	{ txt_help_article_show_raw, PAGE_TOGGLE_RAW },
+	{ txt_help_article_toggle_headers, PAGE_TOGGLE_HEADERS },
 #ifdef HAVE_COLOR
 	{ txt_help_global_toggle_color, GLOBAL_TOGGLE_COLOR },
 #endif /* HAVE_COLOR */
@@ -532,6 +591,41 @@ static t_help_page page_help_page[] = {
 	{ NULL, NOT_ASSIGNED }
 };
 
+static t_help_page url_help_page[] = {
+	{ txt_help_title_navi, NOT_ASSIGNED },
+	{ txt_help_global_page_down, GLOBAL_PAGE_DOWN },
+	{ txt_help_global_page_up, GLOBAL_PAGE_UP },
+	{ txt_help_global_line_down, GLOBAL_LINE_DOWN },
+	{ txt_help_global_line_up, GLOBAL_LINE_UP },
+	{ txt_help_global_scroll_down, GLOBAL_SCROLL_DOWN },
+	{ txt_help_global_scroll_up, GLOBAL_SCROLL_UP },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_url_first_url, GLOBAL_FIRST_PAGE },
+	{ txt_help_url_last_url, GLOBAL_LAST_PAGE },
+	{ txt_help_url_goto_url, NOT_ASSIGNED },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_url_ops, NOT_ASSIGNED },
+	{ txt_help_url_select, URL_SELECT },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_url_search_forwards, GLOBAL_SEARCH_SUBJECT_FORWARD },
+	{ txt_help_url_search_backwards, GLOBAL_SEARCH_SUBJECT_BACKWARD },
+	{ txt_help_global_search_repeat, GLOBAL_SEARCH_REPEAT },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_disp, NOT_ASSIGNED },
+	{ txt_help_url_toggle_info_line, GLOBAL_TOGGLE_INFO_LAST_LINE },
+	{ txt_help_empty_line, NOT_ASSIGNED },
+	{ txt_help_title_misc, NOT_ASSIGNED },
+	{ txt_help_select_quit, GLOBAL_QUIT },
+	{ txt_help_global_help, GLOBAL_HELP },
+	{ txt_help_global_toggle_mini_help, GLOBAL_TOGGLE_HELP_DISPLAY },
+	{ txt_help_global_esc, GLOBAL_ABORT },
+	{ txt_help_global_redraw_screen, GLOBAL_REDRAW_SCREEN },
+#ifndef NO_SHELL_ESCAPE
+	{ txt_help_global_shell_escape, GLOBAL_SHELL_ESCAPE },
+#endif /* !NO_SHELL_ESCAPE */
+	{ NULL, NOT_ASSIGNED }
+};
+
 
 static void
 make_help_page(
@@ -539,8 +633,8 @@ make_help_page(
 	const t_help_page *helppage,
 	const struct keylist keys)
 {
-	char *buf = my_malloc(LEN);
-	char *last = my_malloc(LEN);
+	char *buf;
+	char *last;
 	char key[MAXKEYLEN];
 	/*
 	 * length is only needed to pass it to expand_ctrl_chars()
@@ -552,12 +646,15 @@ make_help_page(
 	if (!helppage)
 		return;
 
+	buf = my_malloc(LEN);
+	last = my_malloc(LEN);
+
 	last[0] = '\0';
 
 	while (helppage->helptext) {
 		if (helppage->func == NOT_ASSIGNED) {
 			/*
-			 * as expand_ctrl_chars() may has shrinked buf
+			 * as expand_ctrl_chars() may has shrunk buf
 			 * make sure buf is large enough to contain the helpline
 			 */
 			buf = my_realloc(buf, LEN);
@@ -602,6 +699,10 @@ show_help_page(
 		return;
 
 	switch (level) {
+		case ATTACHMENT_LEVEL:
+			make_help_page(fp, attachment_help_page, attachment_keys);
+			break;
+
 		case ATTRIB_LEVEL:
 			make_help_page(fp, attrib_help_page, option_menu_keys);
 			break;
@@ -630,6 +731,10 @@ show_help_page(
 			make_help_page(fp, page_help_page, page_keys);
 			break;
 
+		case URL_LEVEL:
+			make_help_page(fp, url_help_page, url_keys);
+			break;
+
 		case INFO_PAGER:
 		default: /* should not happen */
 			error_message(2, _(txt_error_unknown_dlevel));
@@ -657,13 +762,46 @@ show_mini_help(
 		return;
 
 	line = NOTESLINES + MINI_HELP_LINES - 2;
-	bufs = (size_t) MIN((unsigned) cCOLS, (sizeof(buf) - 1));
+	bufs = sizeof(buf) - 1;
 
 #ifdef HAVE_COLOR
 	fcol(tinrc.col_minihelp);
 #endif /* HAVE_COLOR */
 
 	switch (level) {
+		case ATTACHMENT_LEVEL:
+			snprintf(buf, bufs, _(txt_mini_attachment_1),
+				printascii(key[0], func_to_key(GLOBAL_LINE_DOWN, attachment_keys)),
+				printascii(key[1], func_to_key(GLOBAL_LINE_UP, attachment_keys)),
+				printascii(key[2], func_to_key(GLOBAL_HELP, attachment_keys)),
+				printascii(key[3], func_to_key(GLOBAL_QUIT, attachment_keys)));
+			center_line(line, FALSE, buf);
+#ifndef DONT_HAVE_PIPING
+			snprintf(buf, bufs, _(txt_mini_attachment_2),
+				printascii(key[0], func_to_key(ATTACHMENT_SELECT, attachment_keys)),
+				printascii(key[5], func_to_key(ATTACHMENT_PIPE, attachment_keys)),
+				printascii(key[6], func_to_key(GLOBAL_PIPE, attachment_keys)),
+				printascii(key[1], func_to_key(ATTACHMENT_SAVE, attachment_keys)),
+				printascii(key[2], func_to_key(ATTACHMENT_TAG, attachment_keys)),
+				printascii(key[3], func_to_key(ATTACHMENT_TAG_PATTERN, attachment_keys)),
+				printascii(key[4], func_to_key(ATTACHMENT_UNTAG, attachment_keys)));
+#else
+			snprintf(buf, bufs, _(txt_mini_attachment_2),
+				printascii(key[0], func_to_key(ATTACHMENT_SELECT, attachment_keys)),
+				printascii(key[1], func_to_key(ATTACHMENT_SAVE, attachment_keys)),
+				printascii(key[2], func_to_key(ATTACHMENT_TAG, attachment_keys)),
+				printascii(key[3], func_to_key(ATTACHMENT_TAG_PATTERN, attachment_keys)),
+				printascii(key[4], func_to_key(ATTACHMENT_UNTAG, attachment_keys)));
+#endif /* !DONT_HAVE_PIPING */
+			center_line(line + 1, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_attachment_3),
+				printascii(key[1], func_to_key(ATTACHMENT_TOGGLE_TAGGED, attachment_keys)),
+				printascii(key[2], func_to_key(GLOBAL_SEARCH_SUBJECT_FORWARD, attachment_keys)),
+				printascii(key[3], func_to_key(GLOBAL_SEARCH_SUBJECT_BACKWARD, attachment_keys)),
+				printascii(key[4], func_to_key(GLOBAL_SEARCH_REPEAT, attachment_keys)));
+			center_line(line + 2, FALSE, buf);
+			break;
+
 		case SCOPE_LEVEL:
 			snprintf(buf, bufs, _(txt_mini_scope_1),
 				printascii(key[0], func_to_key(SCOPE_ADD, scope_keys)),
@@ -717,19 +855,48 @@ show_mini_help(
 				printascii(key[4], func_to_key(GROUP_MARK_THREAD_READ, group_keys)),
 				printascii(key[5], func_to_key(GROUP_LIST_THREAD, group_keys)));
 			center_line(line + 1, FALSE, buf);
+
+#if defined(DONT_HAVE_PIPING) && defined(DISABLE_PRINTING)
 			snprintf(buf, bufs, _(txt_mini_group_3),
-#ifndef DONT_HAVE_PIPING
-				printascii(key[0], func_to_key(GLOBAL_PIPE, group_keys)),
-#endif /* !DONT_HAVE_PIPING */
-				printascii(key[1], func_to_key(GROUP_MAIL, group_keys)),
-#ifndef DISABLE_PRINTING
-				printascii(key[2], func_to_key(GLOBAL_PRINT, group_keys)),
-#endif /* !DISABLE_PRINTING */
 				printascii(key[3], func_to_key(GLOBAL_QUIT, group_keys)),
 				printascii(key[4], func_to_key(GROUP_TOGGLE_READ_UNREAD, group_keys)),
 				printascii(key[5], func_to_key(GROUP_SAVE, group_keys)),
 				printascii(key[6], func_to_key(GROUP_TAG, group_keys)),
 				printascii(key[7], func_to_key(GLOBAL_POST, group_keys)));
+#else
+#	ifdef DONT_HAVE_PIPING
+			snprintf(buf, bufs, _(txt_mini_group_3),
+				printascii(key[1], func_to_key(GROUP_MAIL, group_keys)),
+				printascii(key[2], func_to_key(GLOBAL_PRINT, group_keys)),
+				printascii(key[3], func_to_key(GLOBAL_QUIT, group_keys)),
+				printascii(key[4], func_to_key(GROUP_TOGGLE_READ_UNREAD, group_keys)),
+				printascii(key[5], func_to_key(GROUP_SAVE, group_keys)),
+				printascii(key[6], func_to_key(GROUP_TAG, group_keys)),
+				printascii(key[7], func_to_key(GLOBAL_POST, group_keys)));
+#	else
+#		ifdef DISABLE_PRINTING
+			snprintf(buf, bufs, _(txt_mini_group_3),
+				printascii(key[0], func_to_key(GLOBAL_PIPE, group_keys)),
+				printascii(key[1], func_to_key(GROUP_MAIL, group_keys)),
+				printascii(key[3], func_to_key(GLOBAL_QUIT, group_keys)),
+				printascii(key[4], func_to_key(GROUP_TOGGLE_READ_UNREAD, group_keys)),
+				printascii(key[5], func_to_key(GROUP_SAVE, group_keys)),
+				printascii(key[6], func_to_key(GROUP_TAG, group_keys)),
+				printascii(key[7], func_to_key(GLOBAL_POST, group_keys)));
+#		else
+			snprintf(buf, bufs, _(txt_mini_group_3),
+				printascii(key[0], func_to_key(GLOBAL_PIPE, group_keys)),
+				printascii(key[1], func_to_key(GROUP_MAIL, group_keys)),
+				printascii(key[2], func_to_key(GLOBAL_PRINT, group_keys)),
+				printascii(key[3], func_to_key(GLOBAL_QUIT, group_keys)),
+				printascii(key[4], func_to_key(GROUP_TOGGLE_READ_UNREAD, group_keys)),
+				printascii(key[5], func_to_key(GROUP_SAVE, group_keys)),
+				printascii(key[6], func_to_key(GROUP_TAG, group_keys)),
+				printascii(key[7], func_to_key(GLOBAL_POST, group_keys)));
+#		endif /* DISABLE_PRINTING */
+#	endif /* DONT_HAVE_PIPING */
+#endif /* DONT_HAVE_PIPING && DISABLE_PRINTING */
+
 			center_line(line + 2, FALSE, buf);
 			break;
 
@@ -762,22 +929,66 @@ show_mini_help(
 				printascii(key[3], func_to_key(PAGE_FOLLOWUP_QUOTE, page_keys)),
 				printascii(key[4], func_to_key(PAGE_MARK_THREAD_READ, page_keys)));
 			center_line(line + 1, FALSE, buf);
+
+#if defined(DONT_HAVE_PIPING) && defined(DISABLE_PRINTING)
 			snprintf(buf, bufs, _(txt_mini_page_3),
-#ifndef DONT_HAVE_PIPING
-				printascii(key[0], func_to_key(GLOBAL_PIPE, page_keys)),
-#endif /* !DONT_HAVE_PIPING */
 				printascii(key[1], func_to_key(PAGE_MAIL, page_keys)),
-#ifndef DISABLE_PRINTING
-				printascii(key[2], func_to_key(GLOBAL_PRINT, page_keys)),
-#endif /* !DISABLE_PRINTING */
 				printascii(key[3], func_to_key(GLOBAL_QUIT, page_keys)),
 				printascii(key[4], func_to_key(PAGE_REPLY_QUOTE, page_keys)),
 				printascii(key[5], func_to_key(PAGE_SAVE, page_keys)),
 				printascii(key[6], func_to_key(PAGE_TAG, page_keys)),
 				printascii(key[7], func_to_key(GLOBAL_POST, page_keys)));
+#else
+#	ifdef DONT_HAVE_PIPING
+			snprintf(buf, bufs, _(txt_mini_page_3),
+				printascii(key[1], func_to_key(PAGE_MAIL, page_keys)),
+				printascii(key[2], func_to_key(GLOBAL_PRINT, page_keys)),
+				printascii(key[3], func_to_key(GLOBAL_QUIT, page_keys)),
+				printascii(key[4], func_to_key(PAGE_REPLY_QUOTE, page_keys)),
+				printascii(key[5], func_to_key(PAGE_SAVE, page_keys)),
+				printascii(key[6], func_to_key(PAGE_TAG, page_keys)),
+				printascii(key[7], func_to_key(GLOBAL_POST, page_keys)));
+#	else
+#		ifdef DISABLE_PRINTING
+			snprintf(buf, bufs, _(txt_mini_page_3),
+				printascii(key[0], func_to_key(GLOBAL_PIPE, page_keys)),
+				printascii(key[1], func_to_key(PAGE_MAIL, page_keys)),
+				printascii(key[3], func_to_key(GLOBAL_QUIT, page_keys)),
+				printascii(key[4], func_to_key(PAGE_REPLY_QUOTE, page_keys)),
+				printascii(key[5], func_to_key(PAGE_SAVE, page_keys)),
+				printascii(key[6], func_to_key(PAGE_TAG, page_keys)),
+				printascii(key[7], func_to_key(GLOBAL_POST, page_keys)));
+#		else
+			snprintf(buf, bufs, _(txt_mini_page_3),
+				printascii(key[0], func_to_key(GLOBAL_PIPE, page_keys)),
+				printascii(key[1], func_to_key(PAGE_MAIL, page_keys)),
+				printascii(key[2], func_to_key(GLOBAL_PRINT, page_keys)),
+				printascii(key[3], func_to_key(GLOBAL_QUIT, page_keys)),
+				printascii(key[4], func_to_key(PAGE_REPLY_QUOTE, page_keys)),
+				printascii(key[5], func_to_key(PAGE_SAVE, page_keys)),
+				printascii(key[6], func_to_key(PAGE_TAG, page_keys)),
+				printascii(key[7], func_to_key(GLOBAL_POST, page_keys)));
+#		endif /* DISABLE_PRINTING */
+#	endif /* DONT_HAVE_PIPING */
+#endif /* DONT_HAVE_PIPING && DISABLE_PRINTING */
+
 			center_line(line + 2, FALSE, buf);
 			break;
 
+		case URL_LEVEL:
+			snprintf(buf, bufs, _(txt_mini_url_1),
+				printascii(key[0], func_to_key(GLOBAL_LINE_DOWN, url_keys)),
+				printascii(key[1], func_to_key(GLOBAL_LINE_UP, url_keys)),
+				printascii(key[2], func_to_key(GLOBAL_HELP, url_keys)),
+				printascii(key[3], func_to_key(GLOBAL_QUIT, url_keys)));
+			center_line(line, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_url_2),
+				printascii(key[2], func_to_key(GLOBAL_SEARCH_SUBJECT_FORWARD, url_keys)),
+				printascii(key[3], func_to_key(GLOBAL_SEARCH_SUBJECT_BACKWARD, url_keys)),
+				printascii(key[4], func_to_key(GLOBAL_SEARCH_REPEAT, url_keys)));
+			center_line(line + 1, FALSE, buf);
+			break;
+
 		case INFO_PAGER:
 			snprintf(buf, bufs, _(txt_mini_info_1),
 				printascii(key[0], func_to_key(GLOBAL_LINE_UP, info_keys)),
