@@ -3,7 +3,7 @@
  *  Module    : main.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2016-02-02
+ *  Updated   : main.c
  *  Notes     :
  *
  * Copyright (c) 1991-2016 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
@@ -308,7 +308,7 @@ main(
 		quick_post_article(post_postponed_and_exit);
 		wait_message(2, _(txt_exiting));
 		no_write = TRUE; /* disable newsrc updates */
-		tin_done(EXIT_SUCCESS);
+		tin_done(EXIT_SUCCESS, NULL);
 	}
 
 	/* TODO: replace hardcoded key-name in txt_info_postponed */
@@ -359,13 +359,13 @@ main(
 	 * Check/start if any new/unread articles
 	 */
 	if (check_any_unread)
-		tin_done(check_start_save_any_news(CHECK_ANY_NEWS, catchup));
+		tin_done(check_start_save_any_news(CHECK_ANY_NEWS, catchup), NULL);
 
 	if (start_any_unread) {
 		batch_mode = TRUE;			/* Suppress some unwanted on-screen garbage */
 		if ((start_groupnum = check_start_save_any_news(START_ANY_NEWS, catchup)) == -1) {
 			batch_mode = FALSE;
-			tin_done(EXIT_SUCCESS);
+			tin_done(EXIT_SUCCESS, NULL);
 		}
 		batch_mode = FALSE;
 	}
@@ -381,7 +381,7 @@ main(
 	 */
 	if (mail_news || save_news) {
 		check_start_save_any_news(mail_news ? MAIL_ANY_NEWS : SAVE_ANY_NEWS, catchup);
-		tin_done(EXIT_SUCCESS);
+		tin_done(EXIT_SUCCESS, NULL);
 	}
 
 	/*
@@ -389,7 +389,7 @@ main(
 	 */
 	if (batch_mode && catchup && !update_index) {
 		catchup_newsrc_file();
-		tin_done(EXIT_SUCCESS);
+		tin_done(EXIT_SUCCESS, NULL);
 	}
 
 	/*
@@ -931,7 +931,7 @@ update_index_files(
 	create_index_lock_file(lock_file);
 	tinrc.thread_articles = THREAD_NONE;	/* stop threading to run faster */
 	do_update(catchup);
-	tin_done(EXIT_SUCCESS);
+	tin_done(EXIT_SUCCESS, NULL);
 }
 
 
