@@ -3,10 +3,10 @@
  *  Module    : memory.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2009-04-14
+ *  Updated   : 2016-08-06
  *  Notes     :
  *
- * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2016 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,6 +49,7 @@ int num_active = -1;
 int max_newnews;
 int num_newnews = 0;
 int max_art;
+int max_base;
 int max_save;
 int num_save = 0;
 int max_scope;
@@ -58,7 +59,7 @@ int num_scope = -1;
  * Dynamic arrays
  */
 int *my_group;				/* .newsrc --> active[] */
-long *base;				/* base articles for each thread */
+t_artnum *base;				/* base articles for each thread */
 struct t_group *active;			/* active newsgroups */
 struct t_scope *scopes = NULL;	/* attributes stores in .tin/attributes */
 struct t_newnews *newnews;		/* active file sizes on differnet servers */
@@ -101,11 +102,12 @@ init_alloc(
 	 * article headers array
 	 */
 	max_art = DEFAULT_ARTICLE_NUM;
+	max_base = DEFAULT_ARTICLE_NUM;
 
-	arts = my_malloc(sizeof(*arts) * max_art);
-	base = my_malloc(sizeof(long) * max_art);
+	arts = my_calloc(1, sizeof(*arts) * max_art);
+	base = my_malloc(sizeof(t_artnum) * max_base);
 
-	ofmt = my_calloc(1, sizeof(*ofmt) * 9);	/* inital number of overview fields */
+	ofmt = my_calloc(1, sizeof(*ofmt) * 9);	/* initial number of overview fields */
 
 	/*
 	 * save file array
@@ -130,9 +132,12 @@ void
 expand_art(
 	void)
 {
+	int i = max_art;
+
 	max_art += max_art >> 1;		/* increase by 50% */
 	arts = my_realloc(arts, sizeof(*arts) * max_art);
-	base = my_realloc(base, sizeof(long) * max_art);
+	for (; i < max_art; i++)		/* use memset() instead? */
+		arts[i].subject = arts[i].from = arts[i].xref = arts[i].refs = arts[i].msgid = NULL;
 }
 
 
@@ -151,6 +156,15 @@ expand_active(
 }
 
 
+void
+expand_base(
+	void)
+{
+	max_base += max_base >> 1;		/* increase by 50% */
+	base = my_realloc(base, sizeof(t_artnum) * max_base);
+}
+
+
 void
 expand_save(
 	void)
@@ -169,7 +183,7 @@ expand_scope(
 			scopes = my_malloc(sizeof(*scopes) * max_scope);
 		num_scope = 0;
 	} else {
-		max_scope += max_scope >> 1;	/* increase by 50 % */
+		max_scope += max_scope >> 1;	/* increase by 50% */
 		scopes = my_realloc(scopes, sizeof(*scopes) * max_scope);
 	}
 }
@@ -237,6 +251,8 @@ free_all_arrays(
 	FreeIfNeeded(quote_regex2.extra);
 	FreeIfNeeded(quote_regex3.re);
 	FreeIfNeeded(quote_regex3.extra);
+	FreeIfNeeded(extquote_regex.re);
+	FreeIfNeeded(extquote_regex.extra);
 #endif /* HAVE_COLOR */
 	FreeIfNeeded(slashes_regex.re);
 	FreeIfNeeded(slashes_regex.extra);
@@ -294,6 +310,8 @@ free_all_arrays(
 	}
 
 	tin_fgets(NULL, FALSE);
+
+	free(tin_progname);
 }
 
 
@@ -304,13 +322,17 @@ free_art_array(
 	int i;
 
 	for_each_art(i) {
-		arts[i].artnum = 0L;
+		arts[i].artnum = T_ARTNUM_CONST(0);
 		arts[i].date = (time_t) 0;
 		FreeAndNull(arts[i].xref);
 
-		/* .refs & .msgid are free()d in build_references() */
-		arts[i].refs = (char *) '\0';
-		arts[i].msgid = (char *) '\0';
+		/*
+		 * .refs & .msgid are usually free()d in build_references()
+		 * nevertheless we try to free() it here in case tin_done()
+		 * was called before build_references()
+		 */
+		FreeAndNull(arts[i].refs);
+		FreeAndNull(arts[i].msgid);
 
 		if (arts[i].archive) {
 			/* ->name is hashed */
@@ -348,6 +370,8 @@ static void
 free_attributes(
 	struct t_attribute *attributes)
 {
+	free_if_not_default(&attributes->group_format, tinrc.group_format);
+	free_if_not_default(&attributes->thread_format, tinrc.thread_format);
 	free_if_not_default(&attributes->date_format, tinrc.date_format);
 	free_if_not_default(&attributes->editor_format, tinrc.editor_format);
 	FreeAndNull(attributes->fcc);
@@ -502,7 +526,7 @@ my_malloc1(
 #endif /* DEBUG */
 
 	if ((p = malloc(size)) == NULL) {
-		error_message(2, txt_out_of_memory, tin_progname, size, file, line);
+		error_message(2, txt_out_of_memory, tin_progname, (unsigned long) size, file, line);
 		giveup();
 	}
 	return p;
@@ -526,7 +550,7 @@ my_calloc1(
 #endif /* DEBUG */
 
 	if ((p = calloc(nmemb, size)) == NULL) {
-		error_message(2, txt_out_of_memory, tin_progname, nmemb * size, file, line);
+		error_message(2, txt_out_of_memory, tin_progname, (unsigned long) (nmemb * size), file, line);
 		giveup();
 	}
 	return p;
@@ -554,7 +578,7 @@ my_realloc1(
 	p = ((!p) ? (malloc(size)) : realloc(p, size));
 
 	if (p == NULL) {
-		error_message(2, txt_out_of_memory, tin_progname, size, file, line);
+		error_message(2, txt_out_of_memory, tin_progname, (unsigned long) size, file, line);
 		giveup();
 	}
 	return p;
