@@ -3,10 +3,10 @@
  *  Module    : page.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2009-09-27
+ *  Updated   : 2016-07-29
  *  Notes     :
  *
- * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2016 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,14 +42,6 @@
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
 
-#if 0
-#if defined(HAVE_IDNA_H) && !defined(_IDNA_H)
-#	include <idna.h>
-#endif /* HAVE_IDNA_H && !_IDNA_H */
-#if defined(HAVE_STRINGPREP_H) && !defined(_STRINGPREP_H)
-#	include <stringprep.h>
-#endif /* HAVE_STRINGPREP_H & !_STRINGPREP_H */
-#endif /* 0 */
 
 /*
  * PAGE_HEADER is the size in lines of the article page header
@@ -63,6 +55,8 @@ static FILE *note_fp;			/* active stream (raw or cooked) */
 static int artlines;			/* active # of lines in pager */
 static t_lineinfo *artline;	/* active 'lineinfo' data */
 
+static t_url *url_list;
+
 t_openartinfo pgart =	/* Global context of article open in the pager */
 	{
 		{ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, FALSE, 0 },
@@ -88,27 +82,38 @@ static int scroll_region_top;	/* first screen line for displayed message */
 static int search_line;			/* Line to commence next search from */
 static t_lineinfo *infoline = (t_lineinfo *) 0;
 
-static t_bool show_all_headers;	/* CTRL-H with headers specified */
+static t_bool show_all_headers;	/* all headers <-> headers in news_headers_to[_not]_display */
+static t_bool show_raw_article;	/* CTRL-H raw <-> cooked article */
 static t_bool reveal_ctrl_l;	/* set when ^L hiding is off */
 
 /*
  * Local prototypes
  */
+static int build_url_list(void);
 static int load_article(int new_respnum, struct t_group *group);
 static int prompt_response(int ch, int curr_respnum);
 static int scroll_page(int dir);
 static t_bool deactivate_next_ctrl_l(void);
 static t_bool activate_last_ctrl_l(void);
+static t_bool process_url(int n);
+static t_bool url_page(void);
 static t_function page_left(void);
 static t_function page_right(void);
 static t_function page_mouse_action(t_function (*left_action) (void), t_function (*right_action) (void));
+static t_function url_left(void);
+static t_function url_right(void);
+static void build_url_line(int i);
 static void draw_page_header(const char *group);
+static void draw_url_arrow(void);
+static void free_url_list(void);
 static void preprocess_info_message(FILE *info_fh);
 static void print_message_page(FILE *file, t_lineinfo *messageline, size_t messagelines, size_t base_line, size_t begin, size_t end, int help_level);
 static void process_search(int *lcurr_line, size_t message_lines, size_t screen_lines, int help_level);
-static void process_url(void);
+static void show_url_page(void);
 static void invoke_metamail(FILE *fp);
 
+static t_menu urlmenu = { 0, 0, 0, show_url_page, draw_url_arrow, build_url_line };
+
 #ifdef XFACE_ABLE
 #	define XFACE_SHOW()	if (tinrc.use_slrnface) \
 								slrnface_show_xface()
@@ -302,14 +307,13 @@ show_page(
 	char buf[LEN];
 	char key[MAXKEYLEN];
 	int i, j, n = 0;
-	int old_sort_art_type = tinrc.sort_article_type;
 	int art_type = GROUP_TYPE_NEWS;
+	int hide_uue_tmp;
+	t_artnum old_artnum = T_ARTNUM_CONST(0);
 	t_bool mouse_click_on = TRUE;
 	t_bool repeat_search;
 	t_function func;
 
-	filtered_articles = FALSE;	/* used in thread level */
-
 	if (group->attribute->mailing_list != NULL)
 		art_type = GROUP_TYPE_MAIL;
 
@@ -326,8 +330,6 @@ show_page(
 	if (srch_lineno != -1)
 		process_search(&curr_line, artlines, ARTLINES, PAGE_LEVEL);
 
-	resize_article(TRUE, &pgart);
-
 	forever {
 		if ((func = handle_keypad(page_left, page_right, page_mouse_action, page_keys)) == GLOBAL_SEARCH_REPEAT) {
 			func = last_search;
@@ -382,7 +384,7 @@ show_page(
 					if (curr_line == 0)
 						info_message(_(txt_begin_of_art));
 					else {
-						curr_line -= (tinrc.scroll_lines == -2) ? ARTLINES / 2 : ARTLINES;
+						curr_line -= ((tinrc.scroll_lines == -2) ? ARTLINES / 2 : ARTLINES);
 						draw_page(group->name, 0);
 					}
 				}
@@ -411,7 +413,7 @@ show_page(
 						if ((func == PAGE_NEXT_UNREAD) && (tinrc.goto_next_unread & GOTO_NEXT_UNREAD_TAB))
 							goto page_goto_next_unread;
 
-						curr_line += (tinrc.scroll_lines == -2) ? ARTLINES / 2 : ARTLINES;
+						curr_line += ((tinrc.scroll_lines == -2) ? ARTLINES / 2 : ARTLINES);
 
 						if (tinrc.scroll_lines == -1)		/* formerly show_last_line_prev_page */
 							curr_line--;
@@ -565,6 +567,10 @@ page_goto_next_unread:
 				}
 				break;
 
+			case GLOBAL_SEARCH_REPEAT:
+				info_message(_(txt_no_prev_search));
+				break;
+
 			case GLOBAL_SEARCH_SUBJECT_FORWARD:	/* search in article */
 			case GLOBAL_SEARCH_SUBJECT_BACKWARD:
 				if (search_article((func == GLOBAL_SEARCH_SUBJECT_FORWARD), repeat_search, search_line, artlines, artline, reveal_ctrl_l_lines, note_fp) == -1)
@@ -629,7 +635,15 @@ page_goto_next_unread:
 				break;
 #endif /* HAVE_PGP_GPG */
 
-			case PAGE_TOGGLE_HEADERS:	/* toggle display of whole 'raw' article */
+			case PAGE_TOGGLE_HEADERS:	/* toggle display of all headers */
+				XFACE_CLEAR();
+				show_all_headers = bool_not(show_all_headers);
+				resize_article(TRUE, &pgart);	/* Also recooks it.. */
+				curr_line = 0;
+				draw_page(group->name, 0);
+				break;
+
+			case PAGE_TOGGLE_RAW:	/* toggle display of whole 'raw' article */
 				XFACE_CLEAR();
 				toggle_raw(group);
 				break;
@@ -678,30 +692,46 @@ page_goto_next_unread:
 
 			case GLOBAL_QUICK_FILTER_SELECT:	/* quickly auto-select article */
 			case GLOBAL_QUICK_FILTER_KILL:		/* quickly kill article */
-				if ((filtered_articles = quick_filter(func, group, &arts[this_resp])))
-					goto return_to_index;
-
-				draw_page(group->name, 0);
+				if (quick_filter(func, group, &arts[this_resp])) {
+					old_artnum = arts[this_resp].artnum;
+					unfilter_articles(group);
+					filter_articles(group);
+					make_threads(group, FALSE);
+					if ((n = find_artnum(old_artnum)) == -1 || which_thread(n) == -1) /* We have lost the thread */
+						return GRP_KILLED;
+					this_resp = n;
+					draw_page(group->name, 0);
+					info_message((func == GLOBAL_QUICK_FILTER_KILL) ? _(txt_info_add_kill) : _(txt_info_add_select));
+				}
 				break;
 
 			case GLOBAL_MENU_FILTER_SELECT:		/* auto-select article menu */
 			case GLOBAL_MENU_FILTER_KILL:			/* kill article menu */
 				XFACE_CLEAR();
 				if (filter_menu(func, group, &arts[this_resp])) {
-					if ((filtered_articles = filter_articles(group)))
-						goto return_to_index;
+					old_artnum = arts[this_resp].artnum;
+					unfilter_articles(group);
+					filter_articles(group);
+					make_threads(group, FALSE);
+					if ((n = find_artnum(old_artnum)) == -1 || which_thread(n) == -1) /* We have lost the thread */
+						return GRP_KILLED;
+					this_resp = n;
 				}
 				draw_page(group->name, 0);
 				break;
 
 			case GLOBAL_EDIT_FILTER:
 				XFACE_CLEAR();
-				if (!invoke_editor(filter_file, filter_file_offset, NULL))
-					break;
-				unfilter_articles();
-				(void) read_filter_file(filter_file);
-				if ((filtered_articles = filter_articles(group)))
-					goto return_to_index;
+				if (invoke_editor(filter_file, filter_file_offset, NULL)) {
+					old_artnum = arts[this_resp].artnum;
+					unfilter_articles(group);
+					(void) read_filter_file(filter_file);
+					filter_articles(group);
+					make_threads(group, FALSE);
+					if ((n = find_artnum(old_artnum)) == -1 || which_thread(n) == -1) /* We have lost the thread */
+						return GRP_KILLED;
+					this_resp = n;
+				}
 				draw_page(group->name, 0);
 				break;
 
@@ -774,7 +804,7 @@ page_goto_next_unread:
 				XFACE_CLEAR();
 				(void) post_response(group->name, this_resp,
 				  (func == PAGE_FOLLOWUP_QUOTE || func == PAGE_FOLLOWUP_QUOTE_HEADERS) ? TRUE : FALSE,
-				  func == PAGE_FOLLOWUP_QUOTE_HEADERS ? TRUE : FALSE, show_all_headers);
+				  func == PAGE_FOLLOWUP_QUOTE_HEADERS ? TRUE : FALSE, show_raw_article);
 				draw_page(group->name, 0);
 				break;
 
@@ -792,21 +822,10 @@ page_goto_next_unread:
 			case GLOBAL_QUIT:	/* return to index page */
 return_to_index:
 				XFACE_CLEAR();
-				if (tinrc.sort_article_type != old_sort_art_type)
-					make_threads(group, TRUE);
-
 				i = which_thread(this_resp);
 				if (threadnum)
 					*threadnum = which_response(this_resp);
 
-				if (filtered_articles) {
-					int old_top = top_art;
-					long old_artnum = arts[this_resp].artnum;
-
-					filter_articles(group);
-					make_threads(group, FALSE);
-					i = find_new_pos(old_top, old_artnum, i);
-				}
 				return i;
 
 			case GLOBAL_TOGGLE_INVERSE_VIDEO:	/* toggle inverse video */
@@ -831,7 +850,13 @@ return_to_index:
 
 			case GLOBAL_OPTION_MENU:	/* option menu */
 				XFACE_CLEAR();
-				config_page(group->name);
+				old_artnum = arts[this_resp].artnum;
+				config_page(group->name, signal_context);
+				if ((this_resp = find_artnum(old_artnum)) == -1 || which_thread(this_resp) == -1) { /* We have lost the thread */
+					pos_first_unread_thread();
+					return GRP_EXIT;
+				}
+				fixup_thread(this_resp, FALSE);
 				draw_page(group->name, 0);
 				break;
 
@@ -885,7 +910,7 @@ return_to_index:
 			case PAGE_REPLY_QUOTE_HEADERS:
 			case PAGE_REPLY:
 				XFACE_CLEAR();
-				mail_to_author(group->name, this_resp, (func == PAGE_REPLY_QUOTE || func == PAGE_REPLY_QUOTE_HEADERS) ? TRUE : FALSE, func == PAGE_REPLY_QUOTE_HEADERS ? TRUE : FALSE, show_all_headers);
+				mail_to_author(group->name, this_resp, (func == PAGE_REPLY_QUOTE || func == PAGE_REPLY_QUOTE_HEADERS) ? TRUE : FALSE, func == PAGE_REPLY_QUOTE_HEADERS ? TRUE : FALSE, show_raw_article);
 				draw_page(group->name, 0);
 				break;
 
@@ -968,18 +993,27 @@ return_to_index:
 
 			case PAGE_VIEW_ATTACHMENTS:
 				XFACE_SUPPRESS();
-				decode_save_mime(&pgart, FALSE);
+				hide_uue_tmp = hide_uue;
+				hide_uue = UUE_NO;
+				resize_article(TRUE, &pgart);
+				attachment_page(&pgart);
+				hide_uue = hide_uue_tmp;
+				resize_article(TRUE, &pgart);
 				draw_page(group->name, 0);
 				XFACE_SHOW();
 				break;
 
 			case PAGE_VIEW_URL:
-				if (!show_all_headers) { /* cooked mode? */
+				if (!show_raw_article) { /* cooked mode? */
+					t_bool success;
+
 					XFACE_SUPPRESS();
 					resize_article(FALSE, &pgart); /* unbreak long lines */
-					process_url();
+					success = url_page();
 					resize_article(TRUE, &pgart); /* rebreak long lines */
 					draw_page(group->name, 0);
+					if (!success)
+						info_message(_(txt_url_done));
 					XFACE_SHOW();
 				}
 				break;
@@ -1014,7 +1048,9 @@ print_message_page(
 			break;
 
 		curr = &messageline[base_line + i];
-		fseek(file, curr->offset, SEEK_SET);
+
+		if (fseek(file, curr->offset, SEEK_SET) != 0)
+			break;
 
 		if ((line = tin_fgets(file, FALSE)) == NULL)
 			break;	/* ran out of message */
@@ -1031,7 +1067,7 @@ print_message_page(
 		/*
 		 * rotN encoding on body and sig data only
 		 */
-		if ((rotate != 0) && ((curr->flags & (C_BODY | C_SIG)) || show_all_headers)) {
+		if ((rotate != 0) && ((curr->flags & (C_BODY | C_SIG)) || show_raw_article)) {
 			for (p = line; *p; p++) {
 				if (*p >= 'A' && *p <= 'Z')
 					*p = (*p - 'A' + rotate) % 26 + 'A';
@@ -1047,7 +1083,7 @@ print_message_page(
 #endif /* !USE_CURSES */
 
 		MoveCursor(i + scroll_region_top, 0);
-		draw_pager_line(line, curr->flags, show_all_headers);
+		draw_pager_line(line, curr->flags, show_raw_article);
 
 		/*
 		 * Highlight URL's and mail addresses
@@ -1168,30 +1204,23 @@ draw_page(
 	if (curr_line + ARTLINES >= artlines) {
 		char buf[LEN];
 		int len;
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-		wchar_t *wbuf;
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 		STRCPY(buf, (arts[this_resp].thread != -1) ? _(txt_next_resp) : _(txt_last_resp));
-#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-		if ((wbuf = char2wchar_t(buf)) != NULL) {
-			len = wcswidth(wbuf, wcslen(wbuf) + 1);
-			free(wbuf);
-		} else
-#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
-			len = (int) strlen(buf);
-
+		len = strwidth(buf);
 		clear_message();
 		MoveCursor(cLINES, cCOLS - len - (1 + BLANK_PAGE_COLS));
+#ifdef HAVE_COLOR
+		fcol(tinrc.col_normal);
+#endif /* HAVE_COLOR */
 		StartInverse();
 		my_fputs(buf, stdout);
-		my_flush();
 		EndInverse();
+		my_flush();
 	} else
 		draw_percent_mark(curr_line + ARTLINES, artlines);
 
 #ifdef XFACE_ABLE
-	if (tinrc.use_slrnface && !show_all_headers)
+	if (tinrc.use_slrnface && !show_raw_article)
 		slrnface_display_xface(note_h->xface);
 #endif /* XFACE_ABLE */
 
@@ -1217,7 +1246,10 @@ invoke_metamail(
 	if (('\0' == *ptr) || (0 == strcmp(ptr, INTERNAL_CMD)) || (NULL != getenv("NOMETAMAIL")))
 		return;
 
-	offset = ftell(fp);
+	if ((offset = ftell(fp)) == -1) {
+		perror_message(_(txt_command_failed), ptr);
+		return;
+	}
 	rewind(fp);
 
 	EndWin();
@@ -1235,18 +1267,18 @@ invoke_metamail(
 #endif /* !DONT_HAVE_PIPING */
 		perror_message(_(txt_command_failed), ptr);
 
+#ifdef USE_CURSES
 	Raw(TRUE);
 	InitWin();
+#endif /* USE_CURSES */
 	prompt_continue();
+#ifndef USE_CURSES
+	Raw(TRUE);
+	InitWin();
+#endif /* !USE_CURSES */
 
 	/* This is needed if we are viewing the raw art */
 	fseek(fp, offset, SEEK_SET);	/* goto old position */
-
-	/* FIXME: values do differ for different languages */
-	MoveCursor(cLINES, cCOLS - 20 - BLANK_PAGE_COLS);
-	StartInverse();
-	my_flush();
-	EndInverse();
 }
 
 
@@ -1263,9 +1295,9 @@ draw_page_header(
 	int len, right_len, center_pos, cur_pos;
 	size_t line_len;
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-	wchar_t *fmt_resp, *fmt_thread, *wtmp, *wtmp2;
+	wchar_t *fmt_resp, *fmt_thread, *wtmp, *wtmp2, *wbuf;
 #else
-	char *tmp;
+	char *tmp, *tmp2;
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	whichresp = which_response(this_resp);
@@ -1324,8 +1356,13 @@ draw_page_header(
 
 	/* group name */
 	if ((wtmp = char2wchar_t(group)) != NULL) {
-		/* wconvert_to_printable(wtmp); */
-		if ((i = wcswidth(wtmp, wcslen(wtmp))) < len)
+		/* wconvert_to_printable(wtmp, FALSE); */
+		if (tinrc.abbreviate_groupname)
+			wtmp2 = abbr_wcsgroupname(wtmp, len);
+		else
+			wtmp2 = wstrunc(wtmp, len);
+
+		if ((i = wcswidth(wtmp2, wcslen(wtmp2))) < len)
 			len = i;
 
 		center_pos = (cCOLS - len) / 2;
@@ -1334,7 +1371,6 @@ draw_page_header(
 		for (; cur_pos < center_pos; cur_pos++)
 			my_fputc(' ', stdout);
 
-		wtmp2 = wstrunc(wtmp, len);
 		my_fputws(wtmp2, stdout);
 		cur_pos += wcswidth(wtmp2, wcslen(wtmp2));
 		free(wtmp2);
@@ -1391,10 +1427,11 @@ draw_page_header(
 
 	/* tex2iso */
 	if (pgart.tex2iso) {
-		wtmp = char2wchar_t(_(txt_tex));
-		my_fputws(wtmp, stdout);
-		cur_pos += wcswidth(wtmp, wcslen(wtmp));
-		free(wtmp);
+		if ((wtmp = char2wchar_t(_(txt_tex))) != NULL) {
+			my_fputws(wtmp, stdout);
+			cur_pos += wcswidth(wtmp, wcslen(wtmp));
+			free(wtmp);
+		}
 	}
 
 	/* subject */
@@ -1406,8 +1443,8 @@ draw_page_header(
 	strncpy(buf, (note_h->subj ? note_h->subj : arts[this_resp].subject), line_len);
 	buf[line_len - 1] = '\0';
 	if ((wtmp = char2wchar_t(buf)) != NULL) {
-		wtmp2 = wstrunc(wtmp, cCOLS - 2 * right_len - 3);
-
+		wbuf = wexpand_tab(wtmp, tabwidth);
+		wtmp2 = wstrunc(wbuf, cCOLS - 2 * right_len - 3);
 		center_pos = (cCOLS - wcswidth(wtmp2, wcslen(wtmp2))) / 2;
 
 		/* pad out to left */
@@ -1420,6 +1457,7 @@ draw_page_header(
 		cur_pos += wcswidth(wtmp2, wcslen(wtmp2));
 		free(wtmp2);
 		free(wtmp);
+		free(wbuf);
 	}
 
 #	ifdef HAVE_COLOR
@@ -1433,10 +1471,11 @@ draw_page_header(
 	if (whichresp)
 		my_printf(_(txt_art_x_of_n), whichresp + 1, x_resp + 1);
 	else {
+		/* TODO: ngettext */
 		if (!x_resp)
-			my_printf(_(txt_no_responses));
+			my_printf("%s", _(txt_no_responses));
 		else if (x_resp == 1)
-			my_printf(_(txt_1_resp));
+			my_printf("%s", _(txt_1_resp));
 		else
 			my_printf(_(txt_x_resp), x_resp);
 	}
@@ -1476,7 +1515,12 @@ draw_page_header(
 		free(wtmp);
 	}
 
-	/* organization */
+	/*
+	 * Organization
+	 *
+	 * TODO: IDNA decoding, see also comment in
+	 *       cook.c:cook_article()
+	 */
 	if ((wtmp = char2wchar_t(_(txt_at_s))) != NULL) {
 		len = wcswidth(wtmp, wcslen(wtmp));
 		free(wtmp);
@@ -1487,8 +1531,8 @@ draw_page_header(
 		snprintf(buf, line_len, _(txt_at_s), note_h->org);
 
 		if ((wtmp = char2wchar_t(buf)) != NULL) {
-			wconvert_to_printable(wtmp);
-			wtmp2 = wstrunc(wtmp, cCOLS - cur_pos - 1);
+			wbuf = wexpand_tab(wtmp, tabwidth);
+			wtmp2 = wstrunc(wbuf, cCOLS - cur_pos - 1);
 
 			i = cCOLS - wcswidth(wtmp2, wcslen(wtmp2)) - 1;
 			for (; cur_pos < i; cur_pos++)
@@ -1497,6 +1541,7 @@ draw_page_header(
 			my_fputws(wtmp2, stdout);
 			free(wtmp2);
 			free(wtmp);
+			free(wbuf);
 		}
 	}
 
@@ -1531,7 +1576,12 @@ draw_page_header(
 	len = cCOLS - 2 * MAX(cur_pos, right_len) - 3;
 
 	/* group name */
-	if ((i = strlen(group)) < len)
+	if (tinrc.abbreviate_groupname)
+		tmp = abbr_groupname(group, len);
+	else
+		tmp = strunc(group, len);
+
+	if ((i = strlen(tmp)) < len)
 		len = i;
 
 	center_pos = (cCOLS - len) / 2;
@@ -1540,7 +1590,6 @@ draw_page_header(
 	for (; cur_pos < center_pos; cur_pos++)
 		my_fputc(' ', stdout);
 
-	tmp = strunc(group, len);
 	my_fputs(tmp, stdout);
 	cur_pos += strlen(tmp);
 	free(tmp);
@@ -1602,7 +1651,8 @@ draw_page_header(
 	strncpy(buf, (note_h->subj ? note_h->subj : arts[this_resp].subject), line_len);
 	buf[line_len - 1] = '\0';
 
-	tmp = strunc(buf, cCOLS - 2 * right_len - 3);
+	tmp2 = expand_tab(buf, tabwidth);
+	tmp = strunc(tmp2, cCOLS - 2 * right_len - 3);
 
 	center_pos = (cCOLS - strlen(tmp)) / 2;
 
@@ -1615,6 +1665,7 @@ draw_page_header(
 	EndInverse();
 	cur_pos += strlen(tmp);
 	free(tmp);
+	free(tmp2);
 
 #	ifdef HAVE_COLOR
 	fcol(tinrc.col_response);
@@ -1627,10 +1678,11 @@ draw_page_header(
 	if (whichresp)
 		my_printf(_(txt_art_x_of_n), whichresp + 1, x_resp + 1);
 	else {
+		/* TODO: ngettext */
 		if (!x_resp)
-			my_printf(_(txt_no_responses));
+			my_printf("%s", _(txt_no_responses));
 		else if (x_resp == 1)
-			my_printf(_(txt_1_resp));
+			my_printf("%s", _(txt_1_resp));
 		else
 			my_printf(_(txt_x_resp), x_resp);
 	}
@@ -1666,12 +1718,14 @@ draw_page_header(
 		/* we have enough space to print at least " at ..." */
 		snprintf(buf, line_len, _(txt_at_s), note_h->org);
 
-		tmp = strunc(buf, cCOLS - cur_pos - 1);
+		tmp2 = expand_tab(buf, tabwidth);
+		tmp = strunc(tmp2, cCOLS - cur_pos - 1);
 		len = cCOLS - (int) strlen(tmp) - 1;
 		for (; cur_pos < len; cur_pos++)
 			my_fputc(' ', stdout);
 		my_fputs(tmp, stdout);
 		free(tmp);
+		free(tmp2);
 	}
 
 	my_fputs(cCRLF, stdout);
@@ -1704,17 +1758,17 @@ load_article(
 #endif /* DEBUG */
 
 	if (new_respnum != this_resp || art_closed) {
-		char *progress_mesg = my_strdup(_(txt_reading_article));
 		int ret;
 
 		art_close(&pgart);			/* close previously opened art in pager */
-
-		ret = art_open(TRUE, &arts[new_respnum], group, &pgart, TRUE, progress_mesg);
-		free(progress_mesg);
+		ret = art_open(TRUE, &arts[new_respnum], group, &pgart, TRUE, _(txt_reading_article));
 
 		switch (ret) {
 			case ART_UNAVAILABLE:
 				art_mark(group, &arts[new_respnum], ART_READ);
+				/* prevent retagging as unread in unfilter_articles() */
+				if (arts[new_respnum].killed == ART_KILLED_UNREAD)
+					arts[new_respnum].killed = ART_KILLED;
 				art_closed = TRUE;
 				wait_message(1, _(txt_art_unavailable));
 				return GRP_ARTUNAVAIL;
@@ -1744,10 +1798,24 @@ load_article(
 				}
 				break;
 		}
+	} else if (show_all_headers) {
+		/*
+		 * article is already opened with show_all_headers ON
+		 * -> re-cook it
+		 */
+		show_all_headers = FALSE;
+		resize_article(TRUE, &pgart);
 	}
 
 	art_mark(group, &arts[this_resp], ART_READ);
 
+	/*
+	 * Change status if art was unread before killing to
+	 * prevent retagging as unread in unfilter_articles()
+	 */
+	if (arts[this_resp].killed == ART_KILLED_UNREAD)
+		arts[this_resp].killed = ART_KILLED;
+
 	if (pgart.cooked == NULL) { /* harmony corruption */
 		wait_message(1, _(txt_art_unavailable));
 		return GRP_ARTUNAVAIL;
@@ -1756,13 +1824,19 @@ load_article(
 	/*
 	 * Setup to start viewing cooked version
 	 */
+	show_raw_article = FALSE;
 	show_all_headers = FALSE;
 	curr_line = 0;
 	note_fp = pgart.cooked;
 	artline = pgart.cookl;
 	artlines = pgart.cooked_lines;
 	search_line = 0;
-	reset_srch_offsets();
+	/*
+	 * Reset offsets only if not invoked during 'body search' (srch_lineno != -1)
+	 * otherwise the found string will not be highlighted
+	 */
+	if (srch_lineno == -1)
+		reset_srch_offsets();
 	rotate = 0;			/* normal mode, not rot13 */
 	reveal_ctrl_l = FALSE;
 	reveal_ctrl_l_lines = -1;	/* all ^L's active */
@@ -1799,7 +1873,7 @@ prompt_response(
 	int ch,
 	int curr_respnum)
 {
-	int num;
+	int i, num;
 
 	clear_message();
 
@@ -1808,7 +1882,10 @@ prompt_response(
 		return -1;
 	}
 
-	return find_response(which_thread(curr_respnum), num);
+	if ((i = which_thread(curr_respnum)) >= 0)
+		return find_response(i, num);
+	else
+		return -1;
 }
 
 
@@ -1869,7 +1946,7 @@ void
 toggle_raw(
 	struct t_group *group)
 {
-	if (show_all_headers) {
+	if (show_raw_article) {
 		artline = pgart.cookl;
 		artlines = pgart.cooked_lines;
 		note_fp = pgart.cooked;
@@ -1975,67 +2052,13 @@ toggle_raw(
 		note_fp = pgart.raw;
 	}
 	curr_line = 0;
-	show_all_headers = bool_not(show_all_headers);
-	draw_page(group->name, 0);
-}
-
-
-static void
-process_url(
-	void)
-{
-	char *ptr;
-	char buf[LEN];
-	char ubuf[LEN];
-	char url[LEN];
-	int i;
-	int offsets[6];
-	int offsets_size = ARRAY_SIZE(offsets);
-
-	/*
-	 * TODO: handle mailto: and news: (not NNTP) URLs internally
-	 */
-	for (i = curr_line; i < artlines; ++i) {
-		if (!(artline[i].flags & (C_URL | C_NEWS | C_MAIL)))
-			continue;
-
-		/*
-		 * Line contains a URL, so read it in
-		 */
-		fseek(pgart.cooked, artline[i].offset, SEEK_SET);
-		ptr = fgets(buf, sizeof(buf), pgart.cooked);
-
-		/*
-		 * Step through, finding URL's
-		 */
-		forever {
-			/* any matches left? */
-			if (pcre_exec(url_regex.re, url_regex.extra, ptr, strlen(ptr), 0, 0, offsets, offsets_size) == PCRE_ERROR_NOMATCH)
-				if (pcre_exec(mail_regex.re, mail_regex.extra, ptr, strlen(ptr), 0, 0, offsets, offsets_size) == PCRE_ERROR_NOMATCH)
-					if (pcre_exec(news_regex.re, news_regex.extra, ptr, strlen(ptr), 0, 0, offsets, offsets_size) == PCRE_ERROR_NOMATCH)
-						break;
-
-			*(ptr + offsets[1]) = '\0';
-
-			if (prompt_default_string("URL:", url, sizeof(url), ptr + offsets[0], HIST_URL)) {
-				if (!*url)			/* Don't try and open nothing */
-					break;
-
-				wait_message(2, _(txt_url_open), url);
-				snprintf(ubuf, sizeof(ubuf), "%s %s", tinrc.url_handler, escape_shell_meta(url, 0));
-				invoke_cmd(ubuf);
-			}
-			ptr += offsets[1] + 1;
-		}
-	}
-	info_message(_(txt_url_done));
+	show_raw_article = bool_not(show_raw_article);
+	draw_page(group ? group->name : "", 0);
 }
 
 
 /*
  * Re-cook an article
- *
- * TODO: check cook_article()s return code
  */
 void
 resize_article(
@@ -2046,9 +2069,10 @@ resize_article(
 	if (artinfo->cooked)
 		fclose(artinfo->cooked);
 
-	cook_article(wrap_lines, artinfo, hide_uue);
+	if (!cook_article(wrap_lines, artinfo, hide_uue, show_all_headers))
+		tin_done(EXIT_FAILURE, _(txt_cook_article_failed_exiting), tin_progname);
 
-	show_all_headers = FALSE;
+	show_raw_article = FALSE;
 	artline = pgart.cookl;
 	artlines = pgart.cooked_lines;
 	note_fp = pgart.cooked;
@@ -2135,7 +2159,7 @@ info_pager(
 					display_info_page(0);
 					break;
 				}
-				curr_info_line += (tinrc.scroll_lines == -2) ? NOTESLINES / 2 : NOTESLINES;
+				curr_info_line += ((tinrc.scroll_lines == -2) ? NOTESLINES / 2 : NOTESLINES);
 				display_info_page(0);
 				break;
 
@@ -2153,7 +2177,7 @@ info_pager(
 					display_info_page(0);
 					break;
 				}
-				curr_info_line -= (tinrc.scroll_lines == -2) ? NOTESLINES / 2 : NOTESLINES;
+				curr_info_line -= ((tinrc.scroll_lines == -2) ? NOTESLINES / 2 : NOTESLINES);
 				display_info_page(0);
 				break;
 
@@ -2274,3 +2298,330 @@ preprocess_info_message(
 	num_info_lines--;
 	infoline = my_realloc(infoline, sizeof(t_lineinfo) * num_info_lines);
 }
+
+
+/*
+ * URL menu
+ */
+static t_function
+url_left(
+	void)
+{
+	return GLOBAL_QUIT;
+}
+
+
+static t_function
+url_right(
+	void)
+{
+	return URL_SELECT;
+}
+
+
+static void
+show_url_page(
+	void)
+{
+	int i;
+
+	signal_context = cURL;
+	currmenu = &urlmenu;
+	mark_offset = 0;
+
+	if (urlmenu.curr < 0)
+		urlmenu.curr = 0;
+
+	ClearScreen();
+	set_first_screen_item();
+	center_line(0, TRUE, _(txt_url_menu));
+
+	for (i = urlmenu.first; i < urlmenu.first + NOTESLINES && i < urlmenu.max; ++i)
+		build_url_line(i);
+
+	show_mini_help(URL_LEVEL);
+
+	draw_url_arrow();
+}
+
+
+static t_bool
+url_page(
+	void)
+{
+	char key[MAXKEYLEN];
+	t_function func;
+	t_menu *oldmenu = NULL;
+
+	if (currmenu)
+		oldmenu = currmenu;
+	urlmenu.curr = 0;
+	urlmenu.max = build_url_list();
+	if (urlmenu.max == 0)
+		return FALSE;
+
+	clear_note_area();
+	show_url_page();
+	set_xclick_off();
+
+	forever {
+		switch ((func = handle_keypad(url_left, url_right, NULL, url_keys))) {
+			case GLOBAL_QUIT:
+				free_url_list();
+				if (oldmenu)
+					currmenu = oldmenu;
+				return TRUE;
+
+			case DIGIT_1:
+			case DIGIT_2:
+			case DIGIT_3:
+			case DIGIT_4:
+			case DIGIT_5:
+			case DIGIT_6:
+			case DIGIT_7:
+			case DIGIT_8:
+			case DIGIT_9:
+				if (urlmenu.max)
+					prompt_item_num(func_to_key(func, url_keys), _(txt_url_select));
+				break;
+
+#ifndef NO_SHELL_ESCAPE
+			case GLOBAL_SHELL_ESCAPE:
+				do_shell_escape();
+				break;
+#endif /* !NO_SHELL_ESCAPE */
+
+			case GLOBAL_HELP:
+				show_help_page(URL_LEVEL, _(txt_url_menu_com));
+				show_url_page();
+				break;
+
+			case GLOBAL_FIRST_PAGE:
+				top_of_list();
+				break;
+
+			case GLOBAL_LAST_PAGE:
+				end_of_list();
+				break;
+
+			case GLOBAL_REDRAW_SCREEN:
+				my_retouch();
+				show_url_page();
+				break;
+
+			case GLOBAL_LINE_DOWN:
+				move_down();
+				break;
+
+			case GLOBAL_LINE_UP:
+				move_up();
+				break;
+
+			case GLOBAL_PAGE_DOWN:
+				page_down();
+				break;
+
+			case GLOBAL_PAGE_UP:
+				page_up();
+				break;
+
+			case GLOBAL_SCROLL_DOWN:
+				scroll_down();
+				break;
+
+			case GLOBAL_SCROLL_UP:
+				scroll_up();
+				break;
+
+			case GLOBAL_TOGGLE_HELP_DISPLAY:
+				toggle_mini_help(URL_LEVEL);
+				show_url_page();
+				break;
+
+			case GLOBAL_TOGGLE_INFO_LAST_LINE:
+				tinrc.info_in_last_line = bool_not(tinrc.info_in_last_line);
+				show_url_page();
+				break;
+
+			case URL_SELECT:
+				if (urlmenu.max) {
+					if (process_url(urlmenu.curr))
+						show_url_page();
+					else
+						draw_url_arrow();
+				}
+				break;
+
+			case GLOBAL_SEARCH_SUBJECT_FORWARD:
+			case GLOBAL_SEARCH_SUBJECT_BACKWARD:
+			case GLOBAL_SEARCH_REPEAT:
+				if (func == GLOBAL_SEARCH_REPEAT && last_search != GLOBAL_SEARCH_SUBJECT_FORWARD && last_search != GLOBAL_SEARCH_SUBJECT_BACKWARD)
+					info_message(_(txt_no_prev_search));
+				else if (urlmenu.max) {
+					int new_pos, old_pos = urlmenu.curr;
+
+					new_pos = generic_search((func == GLOBAL_SEARCH_SUBJECT_FORWARD), (func == GLOBAL_SEARCH_REPEAT), urlmenu.curr, urlmenu.max - 1, URL_LEVEL);
+					if (new_pos != old_pos)
+						move_to_item(new_pos);
+				}
+				break;
+
+			default:
+				info_message(_(txt_bad_command), printascii(key, func_to_key(GLOBAL_HELP, url_keys)));
+				break;
+		}
+	}
+}
+
+
+static void
+draw_url_arrow(
+	void)
+{
+	draw_arrow_mark(INDEX_TOP + urlmenu.curr - urlmenu.first);
+	if (tinrc.info_in_last_line) {
+		t_url *lptr;
+
+		lptr = find_url(urlmenu.curr);
+		info_message("%s", lptr->url);
+	} else if (urlmenu.curr == urlmenu.max - 1)
+		info_message(_(txt_end_of_urls));
+}
+
+
+t_url *
+find_url(
+	int n)
+{
+	t_url *lptr;
+
+	lptr = url_list;
+	while (n-- > 0 && lptr->next)
+		lptr = lptr->next;
+
+	return lptr;
+}
+
+
+static void
+build_url_line(
+	int i)
+{
+	char *sptr;
+	int len = cCOLS - 9;
+	t_url *lptr;
+
+#ifdef USE_CURSES
+	/*
+	 * Allocate line buffer
+	 * make it the same size like in !USE_CURSES case to simplify some code
+	 */
+	sptr = my_malloc(cCOLS + 2);
+#else
+	sptr = screen[INDEX2SNUM(i)].col;
+#endif /* USE_CURSES */
+
+	lptr = find_url(i);
+	snprintf(sptr, cCOLS, "  %s  %-*.*s%s", tin_ltoa(i + 1, 4), len, len, lptr->url, cCRLF);
+	WriteLine(INDEX2LNUM(i), sptr);
+
+#ifdef USE_CURSES
+	free(sptr);
+#endif /* USE_CURSES */
+}
+
+
+static t_bool
+process_url(
+	int n)
+{
+	char *url, *url_esc;
+	size_t len;
+	t_url *lptr;
+
+	lptr = find_url(n);
+	len = strlen(lptr->url) << 1; /* double size; room for editing URL */
+	url = my_malloc(len + 1);
+	if (prompt_default_string("URL:", url, len, lptr->url, HIST_URL)) {
+		if (!*url) {			/* Don't try and open nothing */
+			free(url);
+			return FALSE;
+		}
+		wait_message(2, _(txt_url_open), url);
+		url_esc = escape_shell_meta(url, no_quote);
+		len = strlen(url_esc) + strlen(tinrc.url_handler) + 2;
+		url = my_realloc(url, len);
+		snprintf(url, len, "%s %s", tinrc.url_handler, url_esc);
+		invoke_cmd(url);
+		free(url);
+		cursoroff();
+		return TRUE;
+	}
+	free(url);
+	return FALSE;
+}
+
+
+static int
+build_url_list(
+	void)
+{
+	char *ptr;
+	int i, count = 0;
+	int offsets[6];
+	int offsets_size = ARRAY_SIZE(offsets);
+	t_url *lptr = NULL;
+
+	for (i = 0; i < artlines; ++i) {
+		if (!(artline[i].flags & (C_URL | C_NEWS | C_MAIL)))
+			continue;
+
+		/*
+		 * Line contains a URL, so read it in
+		 */
+		fseek(pgart.cooked, artline[i].offset, SEEK_SET);
+		if ((ptr = tin_fgets(pgart.cooked, FALSE)) == NULL)
+			continue;
+
+		/*
+		 * Step through, finding URL's
+		 */
+		forever {
+			/* any matches left? */
+			if (pcre_exec(url_regex.re, url_regex.extra, ptr, strlen(ptr), 0, 0, offsets, offsets_size) == PCRE_ERROR_NOMATCH)
+				if (pcre_exec(mail_regex.re, mail_regex.extra, ptr, strlen(ptr), 0, 0, offsets, offsets_size) == PCRE_ERROR_NOMATCH)
+					if (pcre_exec(news_regex.re, news_regex.extra, ptr, strlen(ptr), 0, 0, offsets, offsets_size) == PCRE_ERROR_NOMATCH)
+						break;
+
+			*(ptr + offsets[1]) = '\0';
+
+			if (!lptr)
+				lptr = url_list = my_malloc(sizeof(t_url));
+			else {
+				lptr->next = my_malloc(sizeof(t_url));
+				lptr = lptr->next;
+			}
+			lptr->url = my_strdup(ptr + offsets[0]);
+			lptr->next = NULL;
+			++count;
+
+			ptr += offsets[1] + 1;
+		}
+	}
+	return count;
+}
+
+
+static void
+free_url_list(
+	void)
+{
+	t_url *p, *q;
+
+	for (p = url_list; p != NULL; p = q) {
+		q = p->next;
+		free(p->url);
+		free(p);
+	}
+	url_list = NULL;
+}
