@@ -3,9 +3,9 @@
  *  Module    : read.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1997-04-10
- *  Updated   : 2006-02-15
+ *  Updated   : 2016-07-29
  *
- * Copyright (c) 1997-2009 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1997-2016 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -60,17 +60,18 @@ static int offset = 0;
  * local prototypes
  */
 static char *tin_read(char *buffer, size_t len, FILE *fp, t_bool header);
-#ifdef NNTP_ABLE
+#if defined(NNTP_ABLE) && defined(HAVE_SELECT)
 	static t_bool wait_for_input(void);
-#endif /* NNTP_ABLE */
+#endif /* NNTP_ABLE && HAVE_SELECT */
 
 
-#ifdef NNTP_ABLE
+#if defined(NNTP_ABLE) && defined(HAVE_SELECT)
 /*
  * Used by the I/O read routine to look for keyboard input
  * Returns TRUE if user aborted with 'q' or 'z' (lynx-style)
  *         FALSE otherwise
  * TODO: document 'z' (, and allow it's remapping?) and 'Q'
+ *       add a !HAVE_SELECT code path
  */
 static t_bool
 wait_for_input(
@@ -92,9 +93,15 @@ wait_for_input(
 		tv.tv_usec = 0;
 
 /* DEBUG_IO((stderr, "waiting on %d and %d...", STDIN_FILENO, fileno(fd))); */
+#	ifdef HAVE_SELECT_INTP
+		if ((nfds = select(STDIN_FILENO + 1, (int *) &readfds, NULL, NULL, &tv)) == -1) {
+#	else
 		if ((nfds = select(STDIN_FILENO + 1, &readfds, NULL, NULL, &tv)) == -1) {
+#	endif /* HAVE_SELECT_INTP */
+
 			if (errno != EINTR) {
 				perror_message("select() failed");
+				free(tin_progname);
 				giveup();
 			} else
 				return FALSE;
@@ -131,13 +138,17 @@ wait_for_input(
 				}
 
 				if (ch == iKeyQuit || ch == 'z' || ch == iKeyAbort) {
+					if (post_article_and_exit)
+						return FALSE;
 					if (prompt_yn(_(txt_read_abort), FALSE) == 1)
 						return TRUE;
 				}
 
 				if (ch == iKeyQuitTin) {
+					if (post_article_and_exit)
+						return FALSE;
 					if (prompt_yn(_(txt_read_exit), FALSE) == 1)
-						tin_done(EXIT_SUCCESS);
+						tin_done(EXIT_SUCCESS, NULL);
 				}
 
 			}
@@ -153,7 +164,7 @@ wait_for_input(
 
 	}
 }
-#endif /* NNTP_ABLE */
+#endif /* NNTP_ABLE && HAVE_SELECT */
 
 
 /*
@@ -171,7 +182,7 @@ tin_read(
 	t_bool header)
 {
 	char *ptr;
-	signed char c;
+	int c;
 	int i;
 #ifdef NNTP_ABLE
 	t_bool check_dot_only_line;
@@ -186,14 +197,16 @@ tin_read(
 	partial_read = FALSE;
 
 #ifdef NNTP_ABLE
+#	ifdef HAVE_SELECT
 	if (wait_for_input()) {			/* Check if okay to read */
 		info_message(_("Aborting read, please wait..."));
 		drain_buffer(fp);
 		clear_message();
 		tin_errno = TIN_ABORT;
-		fflush(stdin);
+		/* fflush(stdin); */
 		return NULL;
 	}
+#	endif /* HAVE_SELECT */
 
 	errno = 0;		/* To check errno after read, clear it here */
 
@@ -306,7 +319,6 @@ tin_fgets(
 	static char *dynbuf = NULL;
 	static int size = 0;
 
-	char *temp, *ptr;
 	int next;
 
 	tin_errno = 0;					/* Clear errors */
@@ -330,8 +342,8 @@ tin_fgets(
 	size = INIT;
 #endif /* 1 */
 
-	if ((ptr = tin_read(dynbuf, size, fp, header)) == NULL)
-		return ptr;
+	if (tin_read(dynbuf, size, fp, header) == NULL)
+		return NULL;
 
 	if (tin_errno != 0) {
 		DEBUG_IO((stderr, _("Aborted read\n")));
@@ -343,9 +355,8 @@ tin_fgets(
 	while (partial_read) {
 		if (next + RCHUNK > size)
 			size = next + RCHUNK;
-		temp = my_realloc(dynbuf, size * sizeof(*dynbuf));
-		dynbuf = temp;
-		temp = tin_read(dynbuf + next, size - next, fp, header); /* What if == 0? */
+		dynbuf = my_realloc(dynbuf, size * sizeof(*dynbuf));
+		(void) tin_read(dynbuf + next, size - next, fp, header); /* What if == NULL? */
 		next += offset;
 
 		if (tin_errno != 0)
