@@ -3,12 +3,12 @@
  *  Module    : refs.c
  *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1996-05-09
- *  Updated   : 2009-11-17
- *  Notes     : Cacheing of message ids / References based threading
+ *  Updated   : 2016-05-02
+ *  Notes     : Caching of message ids / References based threading
  *  Credits   : Richard Hodson <richard@macgyver.tele2.co.uk>
  *              hash_msgid, free_msgid
  *
- * Copyright (c) 1996-2010 Jason Faultless <jason@altarstone.com>
+ * Copyright (c) 1996-2016 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@
 #define REF_SEP	" \t"			/* Separator chars in ref headers */
 
 #ifdef DEBUG
-#	define DEBUG_PRINT(x)	fprintf x
+#	define DEBUG_PRINT(x)	if (dbgfd != NULL) fprintf x
 static FILE *dbgfd;
 #endif /* DEBUG */
 
@@ -166,7 +166,7 @@ valid_msgid(
 	char *msgid)
 {
 	size_t mlen = 0;
-	t_bool at_present = 0;
+	t_bool at_present = FALSE;
 
 	str_trim(msgid);
 	if (!msgid || *msgid != '<')
@@ -222,6 +222,7 @@ add_msgid(
 
 	if (!msgid) {
 		error_message(2, "add_msgid: NULL msgid\n");
+		free_all_arrays();
 		giveup();
 	}
 
@@ -308,13 +309,13 @@ add_msgid(
 			return i;
 		}
 
-		error_message(2, "Error: Impossible combination of conditions !\n");
+		error_message(2, "Error: Impossible combination of conditions!\n");
 		return i;
 	}
 
 #ifdef DEBUG
 	if (debug & DEBUG_REFS)
-		DEBUG_PRINT((dbgfd, "new: %s -> %s\n", msgid, (newparent)?newparent->txt:"None"));
+		DEBUG_PRINT((dbgfd, "new: %s -> %s\n", msgid, (newparent) ? newparent->txt : "None"));
 #endif /* DEBUG */
 
 	/*
@@ -395,14 +396,26 @@ parse_references(
 	if ((ptr = strtok(r, REF_SEP)) == NULL)
 		return NULL;
 
+	/*
+	 * As per RFC 5536, a leading comment is allowed -> skip unknown
+	 * token until we find a valid msgid
+	 *
+	 * TODO: parse these tokens to be sure it is a comment and not
+	 *       a damaged header
+	 */
+	if (!valid_msgid(ptr)) {
+		while ((ptr = strtok(NULL, REF_SEP)) != NULL && !valid_msgid(ptr))
+			;
+	}
+
+	if (ptr == NULL)
+		return NULL;
+
 	/*
 	 * By definition, the head of the thread has no parent
 	 */
 	parent = NULL;
 
-	if (!valid_msgid(ptr))
-		return NULL;
-
 	current = add_msgid(REF_REF, ptr, parent);
 
 	while ((ptr = strtok(NULL, REF_SEP)) != NULL) {
@@ -427,7 +440,7 @@ rearrange_siblings(
 		current = arts[i].refptr;
 
 		if (!current)
-		continue;
+			continue;
 
 		for (; current->sibling == NULL && current->parent != NULL; current = current->parent)
 			;
@@ -711,9 +724,7 @@ dump_msgid_threads(
 
 	for (i = 0; i < MSGID_HASH_SIZE; i++) {
 		if (msgids[i] != NULL) {
-
 			for (ptr = msgids[i]; ptr != NULL; ptr = ptr->next) {
-
 				if (ptr->parent == NULL) {
 					dump_msgid_thread(ptr, 1);
 					fprintf(dbgfd, "\n");
@@ -721,7 +732,6 @@ dump_msgid_threads(
 			}
 		}
 	}
-
 	fprintf(dbgfd, "Dump complete.\n\n");
 }
 #endif /* DEBUG */
@@ -836,7 +846,7 @@ build_thread(
 
 /*
  * Run a new set of threads through the base articles, using the
- * parent / child / sibling  / article pointers in the msgid hash.
+ * parent / child / sibling / article pointers in the msgid hash.
  */
 void
 thread_by_reference(
@@ -850,8 +860,8 @@ thread_by_reference(
 		char file[PATH_LEN];
 
 		joinpath(file, sizeof(file), TMPDIR, "REFS.info");
-		dbgfd = fopen(file, "w");
-		dump_msgid_threads();
+		if ((dbgfd = fopen(file, "w")) != NULL)
+			dump_msgid_threads();
 	}
 #endif /* DEBUG */
 
@@ -868,16 +878,16 @@ thread_by_reference(
 	}
 
 #ifdef DEBUG
-	if (debug & DEBUG_REFS) {
-		fprintf(dbgfd, "Full dump of threading info...\n");
-		fprintf(dbgfd, "%3s %3s %3s %3s : %3s %3s\n", "#", "Par", "Sib", "Chd", "In", "Thd");
+	if ((debug & DEBUG_REFS) && dbgfd != NULL) {
+		DEBUG_PRINT((dbgfd, "Full dump of threading info...\n"));
+		DEBUG_PRINT((dbgfd, "%3s %3s %3s %3s : %3s %3s\n", "#", "Par", "Sib", "Chd", "In", "Thd"));
 
 		for_each_art(i) {
-			fprintf(dbgfd, "%3d %3d %3d %3d : %3d %3d : %.50s %s\n", i,
+			DEBUG_PRINT((dbgfd, "%3d %3d %3d %3d : %3d %3d : %.50s %s\n", i,
 				(arts[i].refptr->parent) ? arts[i].refptr->parent->article : -2,
 				(arts[i].refptr->sibling) ? arts[i].refptr->sibling->article : -2,
 				(arts[i].refptr->child) ? arts[i].refptr->child->article : -2,
-				arts[i].prev, arts[i].thread, arts[i].refptr->txt, arts[i].subject);
+				arts[i].prev, arts[i].thread, arts[i].refptr->txt, arts[i].subject));
 		}
 
 		fclose(dbgfd);
@@ -928,12 +938,14 @@ collate_subjects(
 			 * is that we have to add later threads onto the end of the
 			 * previous thread
 			 */
-			if (
-					(arts[i].subject == arts[j].subject) /* ||
-					(arts[i].archive && arts[j].archive && (arts[i].archive->name == arts[j].archive->name)) */
-			) {
-/*DEBUG_PRINT((dbgfd, "RES: %d is now previous, at end of %d\n", i, j));*/
-
+#if 1
+			if (arts[i].subject == arts[j].subject)
+#else
+			/* see also art.c:thread_by_subject() */
+			if ((arts[i].subject == arts[j].subject) || (arts[i].archive && arts[j].archive && (arts[i].archive->name == arts[j].archive->name)))
+#endif /* 1 */
+			{
+				/* DEBUG_PRINT((dbgfd, "RES: %d is now previous, at end of %d\n", i, j)); */
 				for (art = j; arts[art].thread >= 0; art = arts[art].thread)
 					;
 
@@ -964,7 +976,7 @@ collate_subjects(
  *    maintained.
  * 3) Add rest of References header to the cache. This information is less
  *    reliable than the info added in 2) and is only used to fill in any
- *    gaps in the reference tree - no information is superceded.
+ *    gaps in the reference tree - no information is superseded.
  * 4) free() up the msgid and refs headers once cached
  */
 void
@@ -987,9 +999,12 @@ build_references(
 		char file[PATH_LEN];
 
 		joinpath(file, sizeof(file), TMPDIR, "REFS.dump");
-		dbgfd = fopen(file, "w");
-		SETVBUF(dbgfd, NULL, _IONBF, 0);
-		fprintf(dbgfd, "MSGID phase\n");
+		if ((dbgfd = fopen(file, "w")) != NULL) {
+#	ifdef HAVE_SETVBUF
+			SETVBUF(dbgfd, NULL, _IONBF, 0);
+#	endif /* HAVE_SETVBUF */
+			DEBUG_PRINT((dbgfd, "MSGID phase\n"));
+		}
 	}
 #endif /* DEBUG */
 
@@ -1012,7 +1027,17 @@ build_references(
 			 * TODO: do this in a single pass
 			 */
 			if ((s = strrchr(art->refs, '<')) != NULL) {
-				if (!strcmp(art->msgid, s)) {
+				char *ptr;
+
+				/*
+				 * A comment can occur after another REF_SEP, remove it
+				 *
+				 * TODO: parse it to be sure it is a comment
+				 */
+				if ((ptr = strpbrk(s, REF_SEP)) != NULL)
+					*ptr = '\0';
+
+				if (valid_msgid(s) && !strcmp(art->msgid, s)) {
 					/*
 					 * Remove circular reference to current article
 					 */
@@ -1022,16 +1047,11 @@ build_references(
 #endif /* DEBUG */
 					*s = '\0';
 				}
-			}
-			if (s != NULL) {
-				if (valid_msgid(art->msgid))
+				if (valid_msgid(art->msgid) && valid_msgid(s))
 					art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, s, NULL));
 				*s = '\0';
-			} else {
-				if (valid_msgid(art->msgid))
-					art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, art->refs, NULL));
+			} else
 				FreeAndNull(art->refs);
-			}
 		}
 
 		/*
@@ -1070,7 +1090,7 @@ build_references(
 	}
 
 #ifdef DEBUG
-	if (debug & DEBUG_REFS)
+	if ((debug & DEBUG_REFS) && dbgfd != NULL)
 		fclose(dbgfd);
 #endif /* DEBUG */
 
