@@ -3,10 +3,10 @@
  *  Module    : signal.c
  *  Author    : I.Lea
  *  Created   : 1991-04-01
- *  Updated   : 2009-12-19
+ *  Updated   : 2016-07-29
  *  Notes     : signal handlers for different modes and window resizing
  *
- * Copyright (c) 1991-2010 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2016 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -117,7 +117,7 @@ static const struct {
 	{ SIGINT,	"SIGINT" },	/* ctrl-C */
 #	endif /* SIGINT */
 #	ifdef SIGQUIT
-	{ SIGQUIT,	"SIGQUIT " },	/* ctrl-\ */
+	{ SIGQUIT,	"SIGQUIT" },	/* ctrl-\ */
 #	endif /* SIGQUIT */
 #	ifdef SIGILL
 	{ SIGILL,	"SIGILL" },	/* illegal instruction */
@@ -253,8 +253,7 @@ handle_resize(
 
 	if (cLINES < MIN_LINES_ON_TERMINAL || cCOLS < MIN_COLUMNS_ON_TERMINAL) {
 		ring_bell();
-		wait_message(3, _(txt_screen_too_small_exiting), tin_progname);
-		tin_done(EXIT_FAILURE);
+		tin_done(EXIT_FAILURE, _(txt_screen_too_small_exiting), tin_progname);
 	}
 
 	TRACE(("handle_resize(%d:%d)", signal_context, repaint));
@@ -290,10 +289,12 @@ handle_resize(
 			display_info_page(0);
 			break;
 
+		case cAttachment:
 		case cGroup:
 		case cScope:
 		case cSelect:
 		case cThread:
+		case cURL:
 			ClearScreen();
 			currmenu->redraw();
 			break;
@@ -303,6 +304,27 @@ handle_resize(
 			draw_page(curr_group->name, 0);
 			break;
 
+		case cPost:
+		case cPostCancel:
+			refresh_post_screen(signal_context);
+			break;
+
+		case cPostFup:
+			resize_article(TRUE, &pgart);
+			draw_page(curr_group->name, 0);
+			/*
+			 * Reset signal_context because draw_page()
+			 * sets signal_context to cPage.
+			 */
+			signal_context = cPostFup;
+			refresh_post_screen(signal_context);
+			break;
+
+		case cReconnect:
+			ClearScreen();
+			show_title(tin_progname);
+			break;
+
 		case cMain:
 			break;
 	}
@@ -398,12 +420,15 @@ signal_handler(
  * fatal error but we don't want the "signal handler caught signal"
  * message here
  */
-#ifdef SIGALRM
+#if defined(HAVE_ALARM) && defined(SIGALRM)
 		case SIGALRM:
-			error_message(2, "NNTP connection error. Exiting...");
-			tin_done(NNTP_ERROR_EXIT);
+#	ifdef DEBUG
+			if (debug & DEBUG_NNTP)
+				debug_print_file("NNTP", "get_server() %d sec elapsed without response", tinrc.nntp_read_timeout_secs);
+#	endif /* DEBUG */
+			tin_done(NNTP_ERROR_EXIT, "NNTP connection error. Exiting...");
 			return;
-#endif /* SIGALRM */
+#endif /* HAVE_ALARM && SIGALRM */
 
 #ifdef SIGCHLD
 		case SIGCHLD:
@@ -452,7 +477,7 @@ signal_handler(
 #endif /* SIGTERM */
 #if defined(SIGHUP) || defined(SIGUSR1) || defined(SIGTERM)
 			dangerous_signal_exit = TRUE;
-			tin_done(-sig);
+			tin_done(-sig, NULL);
 			/* NOTREACHED */
 			break;
 #endif /* SIGHUP || SIGUSR1 || SIGTERM */
