@@ -3,10 +3,10 @@
  *  Module    : xref.c
  *  Author    : I. Lea & H. Brugge
  *  Created   : 1993-07-01
- *  Updated   : 2009-04-09
+ *  Updated   : 2016-06-02
  *  Notes     :
  *
- * Copyright (c) 1993-2010 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1993-2016 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -61,13 +61,14 @@ open_overview_fmt_fp(
 	if (read_news_via_nntp && !read_saved_news) {
 		if (!*nntp_caps.over_cmd)
 			return (FILE *) 0;
+
 		if ((nntp_caps.type == CAPABILITIES && nntp_caps.list_overview_fmt) || nntp_caps.type != CAPABILITIES)
 			return (nntp_command("LIST OVERVIEW.FMT", OK_GROUPS, NULL, 0));
-		else
-			return (FILE *) 0;
-	} else
+
+		return (FILE *) 0;
+	}
 #endif /* NNTP_ABLE */
-		return (fopen(overviewfmt_file, "r"));
+	return (fopen(overviewfmt_file, "r"));
 }
 
 
@@ -82,7 +83,7 @@ overview_xref_support(
 	char *ptr;
 	char *p, *q;
 	t_bool supported = FALSE;
-	size_t res_fields = 9; /* inital number of overview fields */
+	size_t res_fields = 9; /* initial number of overview fields */
 	size_t fields = 0;
 	size_t i;
 
@@ -96,7 +97,7 @@ overview_xref_support(
 
 #if defined(DEBUG) && defined(NNTP_ABLE)
 			if (debug & DEBUG_NNTP)
-				debug_print_file("NNTP", "<<< %s", ptr);
+				debug_print_file("NNTP", "<<<%s%s", logtime(), ptr);
 #endif /* DEBUG && NNTP_ABLE */
 
 			fields++;
@@ -117,7 +118,7 @@ overview_xref_support(
 							expensive_over_parse = TRUE;
 #ifdef DEBUG
 							if (debug & DEBUG_NNTP)
-								debug_print_file("NNTP", "OVERVIEW.FTM: %s at position %d expected %d", ptr, fields, 7);
+								debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 7);
 #endif /* DEBUG */
 						}
 						continue;
@@ -130,7 +131,7 @@ overview_xref_support(
 							expensive_over_parse = TRUE;
 #ifdef DEBUG
 							if (debug & DEBUG_NNTP)
-								debug_print_file("NNTP", "OVERVIEW.FTM: %s at position %d expected %d", ptr, fields, 6);
+								debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 6);
 #endif /* DEBUG */
 						}
 						continue;
@@ -149,9 +150,16 @@ overview_xref_support(
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
 						if (debug & DEBUG_NNTP)
-							debug_print_file("NNTP", "OVERVIEW.FTM: %s at position %d expected > %d", ptr, fields, 7);
+							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected > %d", ptr, fields, 7);
 #endif /* DEBUG */
 					}
+#ifdef DEBUG
+					else {
+						if (debug & DEBUG_NNTP)
+							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d", ptr, fields);
+					}
+#endif /* DEBUG */
+
 					continue;
 				}
 
@@ -163,7 +171,7 @@ overview_xref_support(
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
 						if (debug & DEBUG_NNTP)
-							debug_print_file("NNTP", "OVERVIEW.FTM: %s at position %d expected %d", ptr, fields, 1);
+							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 1);
 #endif /* DEBUG */
 					}
 					continue;
@@ -176,7 +184,7 @@ overview_xref_support(
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
 						if (debug & DEBUG_NNTP)
-							debug_print_file("NNTP", "OVERVIEW.FTM: %s at position %d expected %d", ptr, fields, 2);
+							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 2);
 #endif /* DEBUG */
 					}
 					continue;
@@ -189,7 +197,7 @@ overview_xref_support(
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
 						if (debug & DEBUG_NNTP)
-							debug_print_file("NNTP", "OVERVIEW.FTM: %s at position %d expected %d", ptr, fields, 3);
+							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 3);
 #endif /* DEBUG */
 					}
 					continue;
@@ -202,7 +210,7 @@ overview_xref_support(
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
 						if (debug & DEBUG_NNTP)
-							debug_print_file("NNTP", "OVERVIEW.FTM: %s at position %d expected %d", ptr, fields, 4);
+							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 4);
 #endif /* DEBUG */
 					}
 					continue;
@@ -215,7 +223,7 @@ overview_xref_support(
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
 						if (debug & DEBUG_NNTP)
-							debug_print_file("NNTP", "OVERVIEW.FTM: %s at position %d expected %d", ptr, fields, 5);
+							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 5);
 #endif /* DEBUG */
 					}
 					continue;
@@ -228,7 +236,7 @@ overview_xref_support(
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
 						if (debug & DEBUG_NNTP)
-							debug_print_file("NNTP", "OVERVIEW.FTM: %s at position %d expected %d", ptr, fields, 6);
+							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 6);
 #endif /* DEBUG */
 					}
 					continue;
@@ -241,7 +249,7 @@ overview_xref_support(
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
 						if (debug & DEBUG_NNTP)
-							debug_print_file("NNTP", "OVERVIEW.FTM: %s at position %d expected %d", ptr, fields, 7);
+							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 7);
 #endif /* DEBUG */
 					}
 					continue;
@@ -265,7 +273,7 @@ overview_xref_support(
 	if (fields < 2) {
 #ifdef DEBUG
 		if (debug & DEBUG_NNTP)
-			debug_print_file("NNTP", "OVERVIEW.FTM: Empty response - using safe defaults");
+			debug_print_file("NNTP", fp ? "OVERVIEW.FMT: Empty response - using safe defaults" : "OVERVIEW.FMT: not advertized - using safe defaults");
 #endif /* DEBUG */
 		ofmt = my_realloc(ofmt, sizeof(struct t_overview_fmt) * (8 + 1));
 		ofmt[1].type = OVER_T_STRING;
@@ -303,7 +311,9 @@ overview_xref_support(
 	 *       - in the NNTP_ABLE but !read_news_via_nntp case when
 	 *         OVERVIEW.FMT-file wasn't found or didn't mention Xref:
 	 *       - if the used command is OVER instead of XOVER
+	 *         (use nntp_caps.over_cmd in txt_warn_xref_not_supported ?)
 	 *       - if the used command is HDR XREF instead of XHDR XREF
+	 *         (use nntp_caps.hdr_cmd in txt_warn_xref_not_supported ?)
 	 *       - if server doesn't mention XREF in LIST HEADERS
 	 */
 	if (read_news_via_nntp && !supported)
@@ -324,7 +334,7 @@ art_mark_xref_read(
 	char *xref_ptr;
 	char *groupname;
 	char *ptr, c;
-	long artnum;
+	t_artnum artnum;
 	struct t_group *group;
 #ifdef DEBUG
 	char *debug_mesg;
@@ -356,7 +366,7 @@ art_mark_xref_read(
 			break;
 
 		ptr = xref_ptr++;
-		artnum = atol(xref_ptr);
+		artnum = atoartnum(xref_ptr);
 		while (isdigit((int) *xref_ptr))
 			xref_ptr++;
 
@@ -369,7 +379,7 @@ art_mark_xref_read(
 
 #ifdef DEBUG
 		if (debug & DEBUG_NEWSRC) {
-			debug_mesg = fmt_string("LOOKUP Xref: [%s:%ld] active=[%s] num_unread=[%ld]",
+			debug_mesg = fmt_string("LOOKUP Xref: [%s:%"T_ARTNUM_PFMT"] active=[%s] num_unread=[%"T_ARTNUM_PFMT"]",
 				groupname, artnum,
 				(group ? group->name : ""),
 				(group ? group->newsrc.num_unread : 0));
@@ -388,7 +398,7 @@ art_mark_xref_read(
 						group->newsrc.num_unread--;
 #ifdef DEBUG
 					if (debug & DEBUG_NEWSRC) {
-						debug_mesg = fmt_string("FOUND!Xref: [%s:%ld] marked READ num_unread=[%ld]",
+						debug_mesg = fmt_string("FOUND!Xref: [%s:%"T_ARTNUM_PFMT"] marked READ num_unread=[%"T_ARTNUM_PFMT"]",
 							groupname, artnum, group->newsrc.num_unread);
 							debug_print_comment(debug_mesg);
 							debug_print_bitmap(group, NULL);
@@ -410,10 +420,10 @@ art_mark_xref_read(
 void
 NSETRNG1(
 	t_bitmap *bitmap,
-	long low,
-	long high)
+	t_artnum low,
+	t_artnum high)
 {
-	long i;
+	t_artnum i;
 
 	if (bitmap == NULL) {
 #ifdef DEBUG
@@ -444,10 +454,10 @@ NSETRNG1(
 void
 NSETRNG0(
 	t_bitmap *bitmap,
-	long low,
-	long high)
+	t_artnum low,
+	t_artnum high)
 {
-	long i;
+	t_artnum i;
 
 	if (bitmap == NULL) {
 		error_message(2, "NSETRNG0() failed. Bitmap == NULL");
